# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from random import randint, randrange
from pyromaths.outils.decimaux import decimaux
from pyromaths.ex import LegacyExercise

def ncotation(A, B, longueur, couleur=""):
    """trace une flèche et inscrit horizontalement, dessous, la longueur entre A et B
    où A et B sont des \node"""
    linecouleur = ""
    if couleur != "":
        linecouleur = ",linecolor = %s" % couleur
        couleur = "\\color{%s}" % couleur
    return u"\\ncline[linestyle=dashed, offset = -1.5, linewidth = 0.4pt %s]{<->}{%s}{%s}  \\Bput{%s %s}" % (linecouleur, A, B, couleur, longueur)

def ncotation_h(A, B, longueur, couleur=""):
    """idem mais au dessus de la flèche"""
    linecouleur = ""
    if couleur != "":
        linecouleur = ",linecolor = %s" % couleur
        couleur = "\\color{%s}" % couleur
    return u"\\ncline[linestyle=dashed, offset = 1.5, linewidth = 0.4pt %s]{<->}{%s}{%s}  \\Aput{%s %s}" % (linecouleur, A, B, couleur, longueur)


def _exo_echelles():
    """À partir d'un plan tracé, déterminer l'échelle et des longueurs déterminées."""

    # Échelle
    echelle = [ 100, 250, 400, 500, 750, 1000][randrange(6)]

    # figure : un plan d'appartement déssiner en psTricks
    a, d = randint(40, 60), randint(36, 45)
    b, c = randint(8, d - 27), randint(9, a - 25)
    xF = a - c
    xE = xF - 6
    yK = b + 9
    yF = b - 6
#     yH = b
    # Calculs des grandeurs réelles en mm !
    reels = [echelle * a, echelle * b, echelle * c, echelle * d]

    plan = [a, b, c, d]
    # choix permet de choisir si on donne a, b, c ou d en énoncé
    choix = randrange(4)
    reponses = ["a", "b", "c", "d"]
    enonce = reponses.pop(choix)

    # sur la deuxième ligne du tableau, on écrit en cm, en gras, les nombres calculés
    tab_reels = [ "\\bf"*(i != choix) + decimaux(reels[i] / 10) for i in range(4)]

    # Pour placer les quatre lettres sur le plan
    cotation_couleur = [["a", ""],
                        ["b", ""],
                        ["c", ""],
                        ["d", ""]]
        # la longueur donnée est tracée en bleu
    cotation_couleur[choix][1] = "enonce"

    # figure PSTricks en fonction des paramètres a, d, xE, XF, yF
    figure = ["\\psset{PointName = none,  PointSymbol = none, unit = 1mm, linewidth = .5pt}",
              "\\definecolor{enonce}{rgb}{0.11,0.56,0.98}",
              "\\begin{pspicture}(-10mm, -10mm)(50mm ,50mm)",
              # le rectangle ABCD
              "\\pstGeonode[CurveType = polygon, linewidth = 1pt](0, 0)A(%s,0)B (%s, %s)C (0, %s)D" % (a, a, d, d),
              # les points permettant de placer les cloisons
              "\\pstGeonode(%s, 0){E1}(%s, %s){E2}(%s, 0){F1}(%s, %s){F2}" % (xE, xE, yF, xF, xF, yF),
              "\\pstGeonode(%s, %s){G1}(%s, %s){G2}(%s, %s){G3}" % (a, b, xF, b, xF, 25),
              "\\pstGeonode(%s, %s){H1}(%s, %s){H2}(%s, %s){H3}(%s, %s){H4}" % (xE, b, xE, 25, xE, yK, 15, yK),
              "\\pstGeonode(0, %s){K2}(7, %s){K1}" % (yK, yK),
              "\\pstGeonode(%s, %s){J1}(%s, %s){J2}(%s, %s){J3}(%s, %s){J4}" % (xE, d - 7, xE, d, xF, d - 7, xF, d),
              # trace les cloisons, limitées par des tirets
              "\\ncline{-|}{E1}{E2}\\ncline{-|}{F1}{F2}",
              "\\ncline{|-}{J1}{J2}\\ncline{|-}{J3}{J4}",
              "\\ncline{|-}{K1}{K2}",
              "\\ncline{G1}{G2}\\ncline{|-|}{G3}{G2}",
              "\\ncline{|-|}{H1}{H2}\\ncline{-|}{H3}{H4}",
              # place les cotations sr la figure, l'énoncé en bleu
              ncotation_h("D", "C", cotation_couleur[0][0], cotation_couleur[0][1]),
              ncotation("B", "G1", cotation_couleur[1][0], cotation_couleur[1][1]),
              ncotation("F1", "B", cotation_couleur[2][0], cotation_couleur[2][1]),
              ncotation_h("A", "D", cotation_couleur[3][0], cotation_couleur[3][1]),
              "\\end{pspicture}"]


    exo = [_(u"\\exercice Sur ce plan, la longueur $%s$ mesure en réalité \\unit[%s]{m} :\n") % (enonce, decimaux(reels[choix] / 1000))] \
          + figure + \
           ["\\begin{enumerate}",
           _(u"\\item Déterminer l'échelle de ce plan."),
           _(u"\\item Déterminer les longueurs réelles $%s$, $%s$ et $%s$.") % (reponses[0], reponses[1], reponses[2]),
            "\\end{enumerate}"]
    cor = [_(u"\\exercice* Sur ce plan, la longueur $%s$ mesure en réalité \\unit[%s]{m} : \n") % (enonce, decimaux(reels[choix] / 1000))] \
           + figure + \
          ["\\begin{enumerate}",
           _(u"\\item Déterminer l'échelle de ce plan.\\par"),
           _(u"Sur le plan, je mesure que $%s=\\unit[%s]{cm}$.\\par") % (enonce, decimaux(plan[choix] / 10)),
           _(u"Or on sait que en réalité $%s = \\unit[%s]{m} = \\unit[%s]{cm}$") % (enonce, decimaux(reels[choix] / 1000), decimaux(reels[choix] / 10)),
           _(u" et  $%s \\div %s = %s$.\\par") % (decimaux(reels[choix]), decimaux(plan[choix]), decimaux(echelle)),
           _(u"L'échelle de ce plan est donc $1/%s^e$.") % echelle,
           _(u"\\item Déterminer les longueurs réelles $%s$, $%s$ et $%s$.\n") % (reponses[0], reponses[1], reponses[2]),
           _(u"Grâce à la question précédente, je peux compléter le tableau :\n"),
           "\\begin{tabular}{|l|c|c|c|c|c}",
           ("\\multicolumn{1}{c}{}" + "&\\multicolumn{1}{c}{$%s$}"*4 + "\\\\") % ("a", "b", "c", "d"),
           "\\cline{1-5}",
           _("Sur le plan (en cm)  & %s & %s & %s & %s &\\rnode{plan1}{}\\\\") % tuple([decimaux(n / 10) for n in plan]),
           "\\cline{1-5}",
           _(u"En réalité (en cm)  & %s & %s & %s & %s &\\rnode{plan2}{}\\\\") % tuple(tab_reels),
           "\\cline{1-5}",
           "\\end{tabular}\n",
           "\\ncbar{->}{plan1}{plan2}\\Aput{$\\times %s$}" % echelle,
           _(u"Pour conclure, on convertit ses longueurs en m :\\par"),
           "$$a = \\unit[%s]{m} \\quad ; \\quad b = \\unit[%s]{m} \\quad ; \\quad c  = \\unit[%s]{m} \\quad ; \\quad d =\\unit[%s]{m}$$"\
                   % tuple([decimaux(n / 1000) for n in reels]),
           "\\end{enumerate}"]

    return exo, cor

class exo_echelles(LegacyExercise):
    """Échelles"""

    tags = ["Cinquième"]
    function = _exo_echelles


def exo_fruits():
    fruit = [_("Cerises"), _("Tomates"), _("Pommes"), _("Poires"), _("Raisin"), _("Oranges")][randrange(6)]
    while 1:
        a, b, c = randint(10, 50) / 10, randint(10, 50) / 10, randint(10, 50) / 10
        if a != b and a != c and b != c:
            break
    tarif = randint(20, 50) / 10
    fruits_c = (fruit, a, b, c)
    fruits_e = (fruit, decimaux(a), decimaux(b), "")
    prix_c = (_("prix"), decimaux(fruits_c[1] * tarif), decimaux(fruits_c[2] * tarif), decimaux(fruits_c[3] * tarif))
    prix_e = (_("prix"), decimaux(fruits_c[1] * tarif), "", decimaux(fruits_c[3] * tarif))

    fruits_c = (fruit, decimaux(a), decimaux(b), decimaux(c))
    tableau_exo = ["\\begin{tabular}{|l|c|c|c|}",
               "\hline",
               _(u"%s (en kg) & %s & %s &  %s  \\\\") % fruits_e,
               "\hline",
               _(u"%s (en \\euro)  & %s &  %s  & %s \\\\") % prix_e,
               "\hline",
               "\\end{tabular}"]
    tableau_cor = ["\\begin{tabular}{|l|c|c|c|}",
               "\hline",
               _(u"%s (en kg) & %s & %s &  \\bf %s  \\\\") % fruits_c,
               "\hline",
               _(u"%s (en \\euro)  & %s &  \\bf %s  & %s \\\\") % prix_c,
               "\hline",
               "\\end{tabular}"]
    exo = [_(u"\\exercice Le prix à payer est proportionnel à la masse de fruits achetés.\\par"),
           _(u"Détermine la valeur des cases vides")]
    cor = [_(u"\\exercice Le prix à payer est proportionnel à la masse de fruits achetés.\\par"),
           _(u"Détermine la valeur des cases vides")]
    exo += ["\n"] + tableau_exo
    cor += ["\n"] + tableau_cor
    cor.append("$$\\frac{%s \\times %s}{%s} = %s \\quad;\\qquad" % (prix_e[1], fruits_e[2], fruits_e[1], prix_c[2]))
    cor.append("\\frac{%s \\times %s}{%s} = %s $$" % (fruits_c[1], prix_c[3], prix_e[1], fruits_c[3]))

    return (exo, cor)



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from math import copysign

from pyromaths.classes.PolynomesCollege import Polynome
from pyromaths.classes.Fractions import Fraction
import random

from pyromaths.ex import Jinja2Exercise
from pyromaths.outils.jinja2utils import facteur


class LimitesSuites(Jinja2Exercise):
    """
    Exercice de terminale : Établir la convergence d’une suite, ou sa divergence vers + ∞ ou – ∞.
    """

    tags = ["Term Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        questions = []
        polynome = [[random.choice([-1, 1]) * random.randint(1, 9), i] for i in range(4)]
        polynome[random.randint(1, 3)][0] = 0
        if polynome[3][0]:
            polynome[2][0] = copysign(polynome[2][0], polynome[3][0])
            polynome[1][0] = copysign(polynome[1][0], polynome[3][0])
        else:
            polynome[1][0] = copysign(polynome[1][0], polynome[2][0])
        random.shuffle(polynome)
        polynome = Polynome(polynome, "n")
        polynome_ord = polynome.ordonne()
        l1 = r"-\infty" if polynome_ord[0][0] < 0 else r"+\infty"
        questions.append({"cas": 1, "polynome": polynome, "polynome_ord": polynome_ord, "l1": l1})

        polynome = [[random.choice([-1, 1]) * random.randint(1, 9), i] for i in range(4)]
        polynome[random.randint(1, 3)][0] = 0
        if polynome[3][0]:
            polynome[2][0] = -copysign(polynome[2][0], polynome[3][0])
            polynome[1][0] = -copysign(polynome[1][0], polynome[3][0])
        else:
            polynome[1][0] = -copysign(polynome[1][0], polynome[2][0])
        random.shuffle(polynome)
        polynome = Polynome(polynome, "n")
        polynome_ord = polynome.ordonne()
        l1 = r"-\infty" if polynome_ord[0][0] < 0 else r"+\infty"
        l2 = r"-\infty" if polynome_ord[1][0] < 0 else r"+\infty"
        degre = polynome.degre()
        questions.append(
            {"cas": 2, "polynome": polynome, "polynome_ord": polynome_ord, "l1": l1, "l2": l2, "degre": degre})

        # Quotient qui diverge vers l'infini
        polynome1 = [[random.choice([-1, 1]) * random.randint(1, 9), i] for i in range(4)]
        polynome1[random.randint(1, 2)][0] = 0
        random.shuffle(polynome1)
        polynome1 = Polynome(polynome1, "n")
        polynome1_ord = polynome1.ordonne()
        l1 = r"-\infty" if polynome1_ord[0][0] < 0 else r"+\infty"
        degre1 = polynome1.degre()
        polynome2 = [[random.choice([-1, 1]) * random.randint(1, 9), i] for i in range(4)]
        polynome2[3][0] = 0
        random.shuffle(polynome2)
        polynome2 = Polynome(polynome2, "n")
        polynome2_ord = polynome2.ordonne()
        l2 = r"-\infty" if polynome2_ord[0][0] < 0 else r"+\infty"
        lim = Fraction(polynome1_ord[0][0], polynome2_ord[0][0]).simplifie()
        degre2 = polynome2.degre()
        questions.append(
            {"cas": 3, "polynome1": polynome1, "polynome1_ord": polynome1_ord, "l1": l1, "degre1": degre1,
             "polynome2": polynome2, "polynome2_ord": polynome2_ord, "l2": l2, "degre2": degre2, "lim": lim})

        # Quotient qui converge vers zéro
        polynome1 = [[random.choice([-1, 1]) * random.randint(1, 9), i] for i in range(4)]
        polynome1[3][0] = 0
        random.shuffle(polynome1)
        polynome1 = Polynome(polynome1, "n")
        polynome1_ord = polynome1.ordonne()
        l1 = r"-\infty" if polynome1_ord[0][0] < 0 else r"+\infty"
        degre1 = polynome1.degre()
        polynome2 = [[random.choice([-1, 1]) * random.randint(1, 9), i] for i in range(4)]
        polynome2[random.randint(1, 2)][0] = 0
        random.shuffle(polynome2)
        polynome2 = Polynome(polynome2, "n")
        polynome2_ord = polynome2.ordonne()
        l2 = r"-\infty" if polynome2_ord[0][0] < 0 else r"+\infty"
        lim = Fraction(polynome1_ord[0][0], polynome2_ord[0][0]).simplifie()
        degre2 = polynome2.degre()
        questions.append(
            {"cas": 4, "polynome1": polynome1, "polynome1_ord": polynome1_ord, "l1": l1, "degre1": degre1,
             "polynome2": polynome2, "polynome2_ord": polynome2_ord, "l2": l2, "degre2": degre2, "lim": lim})

        # Quotient qui converge
        i = random.randint(1, 3)
        polynome1 = [[random.choice([-1, 1]) * random.randint(1, 9), i] for i in range(4)]
        polynome1[i][0] = 0
        random.shuffle(polynome1)
        polynome1 = Polynome(polynome1, "n")
        polynome1_ord = polynome1.ordonne()
        l1 = r"-\infty" if polynome1_ord[0][0] < 0 else r"+\infty"
        degre1 = polynome1.degre()
        polynome2 = [[random.choice([-1, 1]) * random.randint(1, 9), i] for i in range(4)]
        polynome2[i][0] = 0
        random.shuffle(polynome2)
        polynome2 = Polynome(polynome2, "n")
        polynome2_ord = polynome2.ordonne()
        l2 = r"-\infty" if polynome2_ord[0][0] < 0 else r"+\infty"
        lim = Fraction(polynome1_ord[0][0], polynome2_ord[0][0]).simplifie()
        degre2 = polynome2.degre()
        questions.append(
            {"cas": 5, "polynome1": polynome1, "polynome1_ord": polynome1_ord, "l1": l1, "degre1": degre1,
             "polynome2": polynome2, "polynome2_ord": polynome2_ord, "l2": l2, "degre2": degre2, "lim": lim})

        random.shuffle(questions)
        suites = ["u_n", "v_n", "w_n", "s_n", "t_n"]
        for i, q in enumerate(questions):
            q["suite"] = suites[i]
        self.context = {
            "questions": questions,
        }

    @property
    def environment(self):
        environment = super().environment
        environment.filters.update({
            'facteur': facteur,
            'Polynome': Polynome,
        })
        return environment



# Pyromaths
#
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
#
# Copyright (C) 2018 -- Louis Paternault (spalax@gresille.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""Exercice de Terminale ES, spécialité : Déterminer un état stable en utilisant un système."""
from decimal import Decimal
import random

from pyromaths.ex import Jinja2Exercise
from pyromaths.outils.jinja2utils import facteur, matrice

# Liste des coefficients de la diagonale de la matrice de transition qui
# donnent des états stables dont la valeur exacte a au plus trois décimales.
CANDIDATS = [
        [".45", ".3 "],
        [".5 ", ".25"],
        [".55", ".2 "],
        [".55", ".25"],
        [".6 ", ".15"],
        [".65", ".1 "],
        [".7 ", ".05"],
        [".7 ", ".1 "],
        [".7 ", ".5 "],
        [".7 ", ".55"],
        [".75", ".45"],
        [".8 ", ".4 "],
        [".8 ", ".7 "],
        [".85", ".4 "],
        [".85", ".55"],
        [".85", ".65"],
        [".85", ".75"],
        [".9 ", ".3 "],
        [".95", ".55"],
        [".95", ".65"],
        [".95", ".8 "],
        [".95", ".85"],
        ]


class EtatStableSysteme2(Jinja2Exercise):
    """Recherche d'état stable (avec un système)"""

    tags = ["Term Spé math", "matrice", "système", "graphe probabiliste", "spécialité"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        ab = random.choice(CANDIDATS)
        random.shuffle(ab)
        self.context = {
            "a": Decimal(ab[0]),
            "b": Decimal(ab[1]),
            }

    @property
    def environment(self):
        environment = super().environment
        environment.filters.update({
            'facteur': facteur,
            })
        return environment

class InterpolationMatrices(Jinja2Exercise):
    """Interpolation polynomiale en utilisant des matrices"""
    # Plus ou moins inspiré du sujet de bac ES Amérique du Nord, juin 2015.

    tags = ["Term Spé math", "matrice", "système", "interpolation", "spécialité"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        X = [None, None, None]
        while len(set(X)) != 3:
            X = sorted([
                random.randint(2, 9),
                random.randint(2, 9),
                random.randint(2, 9),
                ])

        a = b = c = 0
        while True:
            a = Decimal(random.choice([1, -1]) * random.randint(2, 19))
            b = Decimal(random.choice([1, -1]) * random.randint(2, 19))
            c = Decimal(random.choice([1, -1]) * random.randint(2, 19))
            if random.randint(0, 1) == 1:
                a = a / 10
                b = b / 10
                c = c / 10

            if len(set([a, b, c])) != 3:
                continue
            if 10 in (abs(a), abs(b), abs(c)):
                continue
            if 1 in (abs(a), abs(b), abs(c)):
                continue
            break

        M = [
            [X[0]**2, X[0], 1],
            [X[1]**2, X[1], 1],
            [X[2]**2, X[2], 1],
            ]
        Y = [
            a * X[0]**2 + b * X[0] + c,
            a * X[1]**2 + b * X[1] + c,
            a * X[2]**2 + b * X[2] + c,
            ]

        self.context = {
            "X": X,
            "Y": Y,
            "A": [a, b, c],
            "M": M,
            "x": random.randint(10, 19),
            }

    @property
    def environment(self):
        environment = super().environment
        environment.filters.update({
            'facteur': facteur,
            'matrice': matrice,
            'zip': zip,
            })
        return environment



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import math
from random import choice, randrange
import string, math
from pyromaths.ex import LegacyExercise
from pyromaths.outils.Arithmetique import pgcd, valeur_alea

#
# ------------------- PUISSANCES -------------------


def tex_puissances_0(a):
    if isinstance(a, tuple):
        return '\\cfrac{\\nombre{%s} \\times 10^{%s} \\times \\nombre{%s} \\times 10^{%s}}{\\nombre{%s} \\times \\big( 10^{%s} \\big) ^%s}' % \
               (a[0], a[3], a[1], a[4], a[2], a[5], a[6])


def tex_puissances_1(a):
    if isinstance(a, tuple):
        if a[4] < 0:
            return '\\cfrac{\\nombre{%s} \\times \\nombre{%s}}{\\nombre{%s}} \\times \\cfrac{10^{%s+(%s)}}{10^{%s \\times %s}}' % \
                   (a[0], a[1], a[2], a[3], a[4], a[5], a[6])
        else:
            return '\\cfrac{\\nombre{%s} \\times \\nombre{%s}}{\\nombre{%s}} \\times \\cfrac{10^{%s+%s}}{10^{%s \\times %s}}' % \
                   (a[0], a[1], a[2], a[3], a[4], a[5], a[6])


def tex_puissances_2(a):
    if isinstance(a, tuple):
        if a[0] * a[1] / a[2] == a[0] * a[1] // a[2]:
            if a[5] * a[6] < 0:
                return '\\nombre{%s} \\times 10^{%s-(%s)}' % \
                       verifie_type((a[0] * a[1] // a[2], a[3] + a[4], a[5] * a[6]))
            else:
                return '\\nombre{%s} \\times 10^{%s-%s}' % verifie_type((a[0] * a[1] // a[2], a[3] + a[4], a[5] * a[6]))
        else:
            if a[5] * a[6] < 0:
                return '\\nombre{%s} \\times 10^{%s-(%s)}' % \
                       verifie_type((a[0] * a[1] / a[2], a[3] + a[4], a[5] * a[6]))
            else:
                return '\\nombre{%s} \\times 10^{%s-%s}' % verifie_type((a[0] * a[1] / a[2], a[3] + a[4], a[5] * a[6]))


def tex_puissances_3(a):
    from math import floor, log10
    b = int(floor(log10(a[0] * a[1] / a[2])))
    if isinstance(a, tuple) and b != 0:
        return '\\nombre{%s}  \\times 10^{%s} \\times 10^{%s}' % \
               verifie_type((a[0] * a[1] / a[2] / 10 ** b, b, (a[3] + a[4]) - a[5] * a[6]))


def tex_puissances_4(a):
    from math import floor, log10
    b = int(floor(log10(a[0] * a[1] / a[2])))
    if isinstance(a, tuple):
        return '\\nombre{%s}  \\times 10^{%s}' % verifie_type(
            (a[0] * a[1] / a[2] / 10 ** b, (b + a[3] + a[4]) - a[5] * a[6]))


def verifie_type(
        a):  # verifie si des nombres reels dans le tuple a sont en fait des nombres entiers et change leur type
    list = []
    for i in range(len(a)):
        if str(a[i]).endswith('.0'):
            list.append(int(a[i] + .1))
        else:
            list.append(a[i])
    return tuple(list)


def valeurs_puissances():  # renvoie un tuple contenant les valeurs pour les deux exercices sur les puissances
    from math import floor, log10
    (maxi, emax) = (10, 2)
    while True:
        (b1, b2) = (valeur_alea(2, maxi), valeur_alea(2, maxi))
        (b1, b2) = (b1 // pgcd(b1, b2), b2 // pgcd(b1, b2))
        if b1 != 1 and b2 != 1:
            break
    while True:
        (n1, n2) = ((b1 * valeur_alea(2, maxi)) * 10 ** randrange(-emax, emax),
                    (b2 * valeur_alea(2, maxi)) * 10 ** randrange(-emax, emax))
        n3 = ((b1 * b2) * choice((2, 4, 5, 8))) * 10 ** randrange(-emax, emax)
        if int(floor(log10(n1 * n2 / n3))) != 0 and n1 != 1 and n2 != 1 and n3 != 1:
            break
    (e1, e2, e3, e4) = (valeur_alea(-10, 10), valeur_alea(-10, 10), valeur_alea(2, 10), valeur_alea(2, 5))
    a = verifie_type((n1, n2, n3, e1, e2, e3, e4))
    while True:
        (b1, b2) = (valeur_alea(2, maxi), valeur_alea(2, maxi))
        (b1, b2) = (b1 // pgcd(b1, b2), b2 // pgcd(b1, b2))
        if b1 != 1 and b2 != 1:
            break
    (n1, n2) = ((b1 * valeur_alea(2, maxi)) * 10 ** randrange(-emax, emax + 1),
                (b2 * valeur_alea(2, maxi)) * 10 ** randrange(-emax, emax + 1))
    n3 = ((b1 * b2) * choice((1, 2, 4, 5, 8))) * 10 ** randrange(-emax, emax + 1)
    (e1, e2, e3, e4) = (valeur_alea(-10, 10), valeur_alea(-10, 10), valeur_alea(-10, -2), valeur_alea(2, 5))
    b = verifie_type((n1, n2, n3, e1, e2, e3, e4))
    return a, b


def _tex_puissances():
    sd = str.maketrans('.', ',')  # convertit les . en , (separateur decimal)
    valeurs = valeurs_puissances()
    i = randrange(2)
    exo = ['\\exercice''', u"Calculer les expressions suivantes et donner l'écriture scientifique du résultat."]
    exo.append('\\begin{multicols}{2}\\noindent')
    cor = ['\\exercice*''', u"Calculer les expressions suivantes et donner l'écriture scientifique du résultat."]
    cor.append('\\begin{multicols}{2}\\noindent')
    exo.append(u'\\[ \\thenocalcul = ' + tex_puissances_0(valeurs[i]).translate(sd) + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + tex_puissances_0(valeurs[i]).translate(sd) + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + tex_puissances_1(valeurs[i]).translate(sd) + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + tex_puissances_2(valeurs[i]).translate(sd) + '\\] ')
    if int(math.floor(math.log10(valeurs[i][0] * valeurs[i][1] / valeurs[i][2]))) != 0:
        cor.append(u'\\[ \\thenocalcul = ' + tex_puissances_3(valeurs[i]).translate(sd) + '\\] ')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_puissances_4(valeurs[i]).translate(sd) + '} \\] ')
    exo.append('\\columnbreak\\stepcounter{nocalcul}%')
    cor.append('\\columnbreak\\stepcounter{nocalcul}%')
    exo.append(u'\\[ \\thenocalcul = ' + tex_puissances_0(valeurs[1 - i]).translate(sd) + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + tex_puissances_0(valeurs[1 - i]).translate(sd) + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + tex_puissances_1(valeurs[1 - i]).translate(sd) + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + tex_puissances_2(valeurs[1 - i]).translate(sd) + '\\] ')
    if int(math.floor(math.log10(valeurs[1 - i][0] * valeurs[1 - i][1] / valeurs[1 - i][2]))) != 0:
        cor.append(u'\\[ \\thenocalcul = ' + tex_puissances_3(valeurs[1 - i]).translate(sd) + '\\] ')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_puissances_4(valeurs[1 - i]).translate(sd) + '} \\] ')
    exo.append('\\end{multicols}')
    cor.append('\\end{multicols}')
    return exo, cor


class tex_puissances(LegacyExercise):
    """Puissances"""

    tags = ["Troisième"]
    function = _tex_puissances



"""Chemin de quelques répertoires propres à Pyromaths."""

import os
import sys

import pkg_resources

DATADIR = pkg_resources.resource_filename("pyromaths", "data")
EXODIR = os.path.join(DATADIR, "exercices")
LOCALEDIR = os.path.join(DATADIR, "locale")
HOME = os.path.expanduser("~")

if os.name == 'nt': # Windows
    CONFIGDIR = os.path.join(os.environ['APPDATA'], "pyromaths")
elif sys.platform == "darwin":  # Mac OS X
    CONFIGDIR = os.path.join(HOME, "Library", "Application Support", "Pyromaths")
else: # Linux (et autres ?)
    CONFIGDIR = os.path.join(HOME, ".config", "pyromaths")




#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

# fonction affine 3e
# from math import *
import random
from math import sqrt

from pyromaths.classes.Fractions import Fraction  # fractions pyromaths
from pyromaths.classes.PolynomesCollege import Polynome
from pyromaths.outils import Priorites3
from pyromaths.ex import LegacyExercise
from pyromaths.outils.Affichage import decimaux


def extreme(a, b, xmin, xmax, ymin, ymax):
    # donne les extremités de la droite passant par a et b (coordonnées)
    res = []
    x1 = a[0]
    x2 = b[0]
    y1 = a[1]
    y2 = b[1]
    coef = (y1 - y2) / (x1 - x2)
    if coef != 0:
        xsort1 = x1 + (ymin - y1) / coef  # abscisse du point d'ordonnée ymin
        if xsort1 >= xmin and xsort1 <= xmax and not (xsort1, ymin) in res:
            res.append((xsort1, ymin))
        xsort2 = x2 + (ymax - y2) / coef  # abscisse du point d'ordonnée ymax
        if xsort2 >= xmin and xsort2 <= xmax and not (xsort2, ymax) in res:
            res.append((xsort2, ymax))
        ysort1 = y1 + coef * (xmin - x1)  # ordonnée du point d'abscisse xmin
        if ysort1 >= ymin and ysort1 <= ymax and not (xmin, ysort1) in res:
            res.append((xmin, ysort1))
        ysort2 = y2 + coef * (xmax - x2)  # ordonnée du point d'abscisse xmax
        if ysort2 >= ymin and ysort2 <= ymax and not (xmax, ysort2) in res:
            res.append((xmax, ysort2))
    else:
        res = [(xmin, y1), (xmax, y1)]
    return res


def vecdir(A, B):
    # retourne sous forme de liste le vecteur directeur normé de la droite (AB)
    norm = sqrt((B[0] - A[0]) ** 2 + (B[1] - A[1]) ** 2)
    u = [(B[0] - A[0]) / norm, (B[1] - A[1]) / norm]
    if u[0] < 0:
        u[0] = -u[0]
        u[1] = -u[1]
    return u


def validedroite(A, B):
    """ valide le choix du couple A B pour qu'ils ne soient pas "collés",
    la droite (AB) ne sera ni horizontale ni verticale"""
    if abs(A[0] - B[0]) < 1 and abs(A[1] - B[1]) < 1:
        return False
    if A[0] == B[0] or A[1] == B[1]:
        return False
    if abs(A[0] - B[0]) < 1 or abs(A[1] - B[1]) < 1:
        return False
    return True


def validec(A, B):
    # valide le choix du couple A B pour qu'ils ne soient pas "collés"
    if abs(A[0] - B[0]) < 1 and abs(A[1] - B[1]) < 1:
        return False
    else:
        return True


def doublefleche(A, B):
    # trace une flèche "double" de justification en pointillés
    mid = ((A[0] + B[0]) / 2, (A[1] + B[1]) / 2)
    res1 = "\\psline[linestyle=dashed,linewidth=1.1pt]{->}" + str(A) + str(mid) + '\n '
    res2 = "\\psline[linestyle=dashed,linewidth=1.1pt]{->}" + str(mid) + str(B)
    res = res1 + res2
    if A == B:
        res = ""

    return res


def couple():
    A = (random.randrange(-8, 9) / 2, random.randrange(-8, 9) / 2)
    B = (random.randrange(-8, 9) / 2, random.randrange(-8, 9) / 2)
    while not validec(A, B):
        B = (random.randrange(-8, 9) / 2, random.randrange(-8, 9) / 2)
    return (A, B)


def coupletrace():
    A = (0, random.randrange(-4, 5))
    B = (random.randrange(-4, 5), random.randrange(-4, 5))
    while not validec(A, B):
        B = (random.randrange(-4, 5), random.randrange(-4, 5))
    return (A, B)


def couples():
    # génère 6 points. Chaque couple correspondra a une droite ((AB), (CD) et (EF)).
    A = (random.randrange(-8, 9) / 2, random.randrange(-8, 9) / 2)
    B = (random.randrange(-8, 9) / 2, random.randrange(-8, 9) / 2)
    while not validedroite(A, B):
        B = (random.randrange(-8, 9) / 2, random.randrange(-8, 9) / 2)
    C = (0, random.randrange(-4, 5))
    while not (validec(A, C) and validec(B, C)):
        C = (0, random.randrange(-4, 5))
    D = (random.randrange(-4, 5), random.randrange(-4, 5))
    while not (validec(A, D) and validec(B, D) and validedroite(C, D)):
        D = (random.randrange(-4, 5), random.randrange(-4, 5))
    E = (0, random.randrange(-8, 9) / 2)
    while not (validec(A, E) and validec(B, E) and validec(C, E) and validec(D, E)):
        E = (0, random.randrange(-8, 9) / 2)
    F = (random.randrange(-4, 5), random.randrange(-4, 5))
    while not (validec(A, F) and validec(B, F) and validec(C, F) and validec(D, F) and validedroite(E, F)):
        F = (random.randrange(-4, 5), random.randrange(-4, 5))
    return (A, B, C, D, E, F)


def tracedroite(A, B, xmin, xmax, ymin, ymax):
    # trace la droite (AB)
    l = extreme(A, B, xmin, xmax, ymin, ymax)
    return "\\psline " + str(l[0]) + str(l[1])


def dansrep(A, xmin, xmax, ymin, ymax):
    # Si le points est dans le repère
    res = False
    if A[0] > xmin and A[0] < xmax and A[1] > ymin and A[1] < ymax:
        res = True
    return res


def nomdroite(i, coordo):
    # place le nom de la droite (d_i) sur le graphique aux coordonnées coordo
    x0 = coordo[0]
    y0 = coordo[1]
    if x0 != 0:
        x = x0 / abs(x0) * (abs(x0) + 0.5)
    else:
        x = 0
    if y0 != 0:
        y = y0 / abs(y0) * (abs(y0) + 0.5)
    else:
        y = 0
    return "\\rput" + str((x, y)) + "{($d_" + str(i) + "$)}"


def nom3droites(A, B, C, D, E, F, xmin, xmax, ymin, ymax):
    # place le nom des droites (AB), (CD), (EF) dans le graphique
    # les 3 instructions latex sont contenues dans une liste
    # évite la juxtaposition des écritures
    res = []
    l1 = extreme(A, B, xmin, xmax, ymin, ymax)
    res.append(nomdroite(1, l1[0]))
    l2 = extreme(C, D, xmin, xmax, ymin, ymax)
    if validec(l1[0], l2[0]):
        memo = l2[0]  # se rappeler qu'on a utilisé l2[0]
        res.append(nomdroite(2, l2[0]))
    else:
        memo = l2[1]
        res.append(nomdroite(2, l2[1]))
    l3 = extreme(E, F, xmin, xmax, ymin, ymax)
    if validec(l1[0], l3[0]) and validec(memo, l3[0]):
        res.append(nomdroite(3, l3[0]))
    else:
        res.append(nomdroite(3, l3[1]))
    return res


def isint(x):
    # est entier?
    res = False
    if int(x) == x:
        res = True
    return res


def isdemi(x):
    # est-ce une moitié d'entier
    res = False
    x = x * 2
    if isint(x):
        res = True

    return res


def coefdir(A, B):
    # donne le coefficient directeur x/y sous forme de liste [x,y]
    # Si y=1 on ecrira x sinon on écrira la fraction x/y
    return Fraction.simplifie(Fraction(B[1] - A[1], B[0] - A[0]))


def anteimage(fonc, A, B):
    # Génère la 1ère question et sa réponse

    l = [' l\'image de ', ' un nombre qui a pour image ', u' un antécédent de ']
    lcor = [' est l\'image de ', ' a pour image ', u' est un antécédent de ']  # liste pour le corrigé
    i = random.randrange(0, 2)
    j = i
    if i == 1:
        j = i + random.randrange(0, 2)
    res = []
    res.append('Donner ' + l[j] + '$' + decimaux(A[i]) + '$' + ' par la fonction ' + '\\textit{' + fonc + '}.')
    res.append('$' + decimaux(A[abs(i - 1)]) + '$' + lcor[j] + '$' + decimaux(
        A[i]) + '$' + ' par la \\hbox{fonction ' + '\\textit{' + fonc + '}}.')
    if i == 0:
        res.append(doublefleche((A[0], 0), A))
        res.append(doublefleche(A, (0, A[1])))

    else:
        res.append(doublefleche((0, A[1]), A))
        res.append(doublefleche(A, (A[0], 0)))
    i = abs(i - 1)
    j = i
    if i == 1:
        j = i + random.randrange(0, 2)
    res.append('Donner ' + l[j] + '$' + decimaux(B[i]) + '$' + ' par la fonction ' + '\\textit{' + fonc + '}.')
    res.append('$' + decimaux(B[abs(i - 1)]) + '$' + lcor[j] + '$' + decimaux(
        B[i]) + '$' + ' par la \\hbox{fonction ' + '\\textit{' + fonc + '}}.')
    if i == 0:
        res.append(doublefleche((B[0], 0), B))
        res.append(doublefleche(B, (0, B[1])))
    else:
        res.append(doublefleche((0, B[1]), B))
        res.append(doublefleche(B, (B[0], 0)))
    return res


def tracefonc(f, i, A, B, xmin, xmax, ymin, ymax):
    """**tracefonc\ (*f*\ , *i*\ , *A*\ , *B*\ , *xmin*\ , *xmax*\ , *ymin*\ , *ymax*\ )

        *A* est sur l'axe des ordonnées, *f* est le nom de la fonction
        Génère la 2e queston et sa réponse

        >>> from pyromaths.classes.Fractions import Fraction
        >>> affine.tracefonc('f', 1, (0,-2),(3,2),-4,4,-4,-4)
        [u'Tracer la droite repr\xe9sentative ($d_1$) de la fonction $f:x\\longmapsto \\
            dfrac{4}{3}\\,x-2$.', 'On sait que $f(0)=-2$ et $f(-3)=\\dfrac{4}{3}\\times \\le
            ft( -3\\right) -2=\\dfrac{4\\times \\cancel{3}\\times -1}{\\cancel{3}\\times 1}-
            2=-4-2=-6', '\\psdot [dotsize=4.5pt,dotstyle=x](0, -2)', '\\psdot [dotsize=4.5pt
            ,dotstyle=x](-3, -6.0)']

        :rtype: list of string
        """
    u = coefdir(A, B)
    if isinstance(u, int) or u.d == 1:
        x1 = decimaux(B[0])
    else:
        B = (u.d, u.n + A[1])
        if not dansrep(B, xmin, xmax, ymin, ymax):
            B = (-u.d, -u.n + A[1])
        x1 = decimaux(B[0])
    l = Priorites3.texify([Polynome([[u, 1], [A[1], 0]], "x")(B[0])])
    l.extend(Priorites3.texify(Priorites3.priorites(Polynome([[u, 1], [A[1], 0]], "x")(B[0]))))
    l = [u'Tracer la droite représentative ($d_' + str(i) + '$) de la fonction $' + f + ':x\\longmapsto ' + str(
        Polynome([[u, 1], [A[1], 0]], "x")) + '$.',
         'On sait que $' + f + '(0)=' + decimaux(A[1]) + '$ et $' + f + '(' + x1 + ')=' + "=".join(l) + "$.",
         '\\psdot [dotsize=4.5pt,dotstyle=x]' + str(A),
         '\\psdot [dotsize=4.5pt,dotstyle=x]' + str(B),
         ]
    return l


def exprfonc(f, i, A, B):
    # Génère la 3e question.
    # A est sur l'axe des ordonnées, f est le nom de la fonction
    u = coefdir(A, B)
    if isinstance(u, int): u = Fraction(u, 1)
    Polynome([[u, 1], [A[1], 0]], "x")(B[0])
    if A[1] - B[1] > 0:
        deltay = '+' + decimaux(A[1] - B[1])
    else:
        deltay = decimaux(A[1] - B[1])
    if A[0] - B[0] > 0:
        deltax = '+' + decimaux(A[0] - B[0])
    else:
        deltax = decimaux(A[0] - B[0])

    if B[0] < 0:
        mid11 = B[0] - 0.75
        mid12 = (B[1] + A[1]) / 2  # milieu de la flèche verticale
    else:
        mid11 = B[0] + 0.75
        mid12 = (B[1] + A[1]) / 2
    if B[0] * u.d / u.n > 0:
        mid21 = (A[0] + B[0]) / 2
        mid22 = A[1] - 0.6  # milieu de la flèche horizontale
    else:
        mid21 = (A[0] + B[0]) / 2
        mid22 = A[1] + 0.5
    if mid12 < 0 and mid12 > -0.8:
        mid12 = -1
    if mid12 >= 0 and mid12 < 0.5:
        mid12 = 0.5
    if mid11 < 0 and mid11 > -0.8:
        mid11 = -1
    if mid11 >= 0 and mid11 < 0.5:
        mid11 = 0.5
    if mid21 < 0 and mid21 > -0.8:
        mid21 = -1
    if mid21 >= 0 and mid21 < 0.5:
        mid21 = 0.5
    if mid22 < 0 and mid22 > -0.8:
        mid22 = -1
    if mid22 >= 0 and mid22 < 0.5:
        mid22 = 0.5
    mid1 = (mid11, mid12)
    mid2 = (mid21, mid22)
    l = [u'Déterminer l\'expression de la fonction $' + f + u'$ représentée ci-contre par la droite ($d_' + str(
        i) + '$).',
         u'On lit l\'ordonnée à l\'origine et le coefficient de la fonction affine sur le graphique.\\\ ',
         '$' + f + '(x)=a\\,x+b$ ' + 'avec $b=' + decimaux(
             A[1]) + '$ et $a=' + '\\dfrac{' + deltay + '}{' + deltax + '}=' + str(u) + '$.\\\ ',
         'L\'expression de la fonction $' + f + '$ est $' + f + '(x)=' + str(Polynome([[u, 1], [A[1], 0]], "x")) + '$.',
         doublefleche(B, (B[0], A[1])),
         doublefleche((B[0], A[1]), A),
         '\\rput' + str(mid1) + '{(' + deltay + ')}',
         '\\rput' + str(mid2) + '{(' + deltax + ')}']

    return l


def _affine():
    # Génère l'exercice
    xmin, xmax, ymin, ymax = -5, 5, -5, 5
    f = ['f', 'g', 'h', 'k', 'l', 'u']
    rgfonc1 = random.randrange(0, 6)
    fonc1 = f[rgfonc1]
    (A, B, C, D, E, F) = couples()
    l = anteimage(fonc1, A, B)  # lecture d'image d'antécédent
    fonc2 = f[(rgfonc1 + 1) % 6]
    l2 = tracefonc(fonc2, 2, C, D, xmin, xmax, ymin, ymax)  # représenter une fonction
    fonc3 = f[(rgfonc1 + 2) % 6]
    l3 = exprfonc(fonc3, 3, E, F)
    noms = nom3droites(A, B, C, D, E, F, xmin, xmax, ymin, ymax)
    exo = ["\\exercice", "\\parbox{0.5\\linewidth}{",
           u"($d_1$) est la droite représentative de la fonction $" + fonc1 + "$.",
           "\\begin{enumerate}",
           "\\item " + l[0],
           "\\item " + l[4],
           "\\item " + l2[0],
           "\\item " + l3[0],
           "\\end{enumerate}}\\hfill",
           "\\parbox{0.45\\linewidth}{",
           "\\psset{unit=0.8cm}",
           "\\begin{pspicture}" + str((xmin, ymin)) + str((xmax, ymax)),
           "\\psgrid[subgriddiv=2, gridlabels=8pt](0,0)" + str((xmin, ymin)) + str((xmax, ymax)),
           "\\psline[linewidth=1.2pt]{->}" + str((xmin, 0)) + str((xmax, 0)),
           "\\psline[linewidth=1.2pt]{->}" + str((0, ymin)) + str((0, ymax)),
           tracedroite(A, B, xmin, xmax, ymin, ymax),
           noms[0],
           tracedroite(E, F, xmin, xmax, ymin, ymax),
           noms[2],
           "\\end{pspicture}}"]

    cor = ["\\exercice*", "\\setlength{\\columnsep}{2mm}",
           "\\begin{multicols}{2}\\noindent \\small",
           u"($d_1$) est la droite représentative de la fonction $" + fonc1 + "$.",
           "\\begin{enumerate}",
           "\\item " + l[1],
           "\\item " + l[5],
           "\\item\n\\begin{flushleft}\n" + l2[1] + "\n\\end{flushleft}",
           "\\item\n\\begin{flushleft}\n" + l3[1],
           l3[2],
           l3[3] + "\n\\end{flushleft}",
           "\\end{enumerate}",
           "\\end{multicols}",
           "\\vspace{0.45cm}",
           "\\begin{minipage}{0.5\\linewidth}",
           "\\psset{unit=0.7cm}",
           "\\begin{center}",
           "\\begin{pspicture}" + str((xmin, ymin)) + str((xmax, ymax)),
           "\\psgrid[subgriddiv=2, gridlabels=8pt](0,0)" + str((xmin, ymin)) + str((xmax, ymax)),
           "\\psline[linewidth=1.2pt]{->}" + str((xmin, 0)) + str((xmax, 0)),
           "\\psline[linewidth=1.2pt]{->}" + str((0, ymin)) + str((0, ymax)),
           tracedroite(A, B, xmin, xmax, ymin, ymax),
           noms[0],
           tracedroite(C, D, xmin, xmax, ymin, ymax),
           noms[1],
           l[2],
           l[3],
           l[6],
           l[7],
           l2[2],
           l2[3],
           "\\end{pspicture}",
           "\\end{center}",
           "\\end{minipage}",
           "\\begin{minipage}{0.5\\linewidth}",
           "\\psset{unit=0.7cm}",
           "\\begin{center}",
           "\\begin{pspicture}" + str((xmin, ymin)) + str((xmax, ymax)),
           "\\psgrid[subgriddiv=2, gridlabels=8pt](0,0)" + str((xmin, ymin)) + str((xmax, ymax)),
           "\\psline[linewidth=1.2pt]{->}" + str((xmin, 0)) + str((xmax, 0)),
           "\\psline[linewidth=1.2pt]{->}" + str((0, ymin)) + str((0, ymax)),
           tracedroite(E, F, xmin, xmax, ymin, ymax),
           noms[2],
           l3[4],
           l3[5],
           l3[6],
           l3[7],
           "\\end{pspicture}",
           "\\end{center}",
           "\\end{minipage}",
           "\\vspace{0.45cm}"]
    return exo, cor

class affine(LegacyExercise):
    """Fonctions affines"""

    tags = ["Troisième"]
    function = _affine



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from pyromaths.outils.Arithmetique import pgcd, valeur_alea
from pyromaths.outils.Affichage import decimaux, tex_coef
from random import choice, randrange
from pyromaths.ex import LegacyExercise
import string


def tex_proprietes_val(exp_max, nb_max, typeexo):
    """
    Renvoie des valeurs pour l'exercice sur les propriétés des puissances
    @param exp_max: valeur maximale pour les exposants
    @type exp_max: Integer
    @param nb_max: Valeur maximale pour les nombres
    @type nb_max: integer
    @param typeexo: 0 : 2 exposants et 1 nombre ; 1 : 1 exposant et 2 nombres
    @type typeexo: integer
    """

    if typeexo:
        while 1:
            nb1 = randrange(2, nb_max)
            nb2 = randrange(2, nb_max)
            exp1 = randrange(2, exp_max)
            exp2 = exp1
            if nb1 != nb2:
                break
    else:
        while 1:
            nb1 = randrange(2, nb_max)
            nb2 = nb1
            exp1 = randrange(2, exp_max)
            exp2 = randrange(2, exp_max)
            if exp1 != exp2:
                break
    return (nb1, exp1, nb2, exp2)


def _tex_proprietes():
    exo = ["\\exercice",
           u"Compléter par un nombre de la forme $a^n$ avec $a$ et $n$ entiers :",
           "\\begin{multicols}{4}", "  \\noindent%",
           "  \\begin{enumerate}"]
    cor = ["\\exercice*",
           u"Compléter par un nombre de la forme $a^n$ avec $a$ et $n$ entiers :",
           "\\begin{multicols}{4}", "  \\noindent%",
           "  \\begin{enumerate}"]
    lexos = [0, 1, 2, 3, 0, 1, 2, 3]

    # 0: a^n*a^p ; 1: (a^n)^p ; 2:a^n/a^p ; 3: a^n*b^n

    for dummy in range(len(lexos)):
        j = lexos.pop(randrange(len(lexos)))
        if j == 3:
            lval = tex_proprietes_val(12, 12, 1)
            exo.append("\\item $%s^{%s} \\times %s^{%s} = \\dotfill$" % 
                       lval)
            cor.append("\\item $%s^{%s}\\times%s^{%s}=" % lval)
            cor.append("%s^{%s}$" % (lval[0] * lval[2], lval[1]))
        else:
            lval = tex_proprietes_val(12, 12, 0)
            if j == 0:
                exo.append("\\item $%s^{%s}\\times%s^{%s}=\\dotfill$" % 
                           lval)
                cor.append("\\item $%s^{%s}\\times%s^{%s}=" % lval)
                cor.append("%s^{%s}$" % (lval[0], lval[1] + lval[3]))
            elif j == 1:
                exo.append("\\item $(%s^{%s})^{%s}=\\dotfill$" % (lval[0],
                           lval[1], lval[3]))
                cor.append("\\item $(%s^{%s})^{%s}=" % (lval[0],
                           lval[1], lval[3]))
                cor.append("%s^{%s}$" % (lval[0], lval[1] * lval[3]))
            elif j == 2:
                while lval[1] - lval[3] < 3:
                    lval = tex_proprietes_val(12, 12, 0)
                exo.append("\\item $\\dfrac{%s^{%s}}{%s^{%s}}=\\dotfill$" % 
                           lval)
                cor.append("\\item $\\dfrac{%s^{%s}}{%s^{%s}}=" % 
                           lval)
                cor.append("%s^{%s}$" % (lval[0], lval[1] - lval[3]))
    exo.append("\\end{enumerate}")
    exo.append("\\end{multicols}\n")
    cor.append("\\end{enumerate}")
    cor.append("\\end{multicols}\n")
    return (exo, cor)

class tex_proprietes(LegacyExercise):
    """Propriétés sur les puissances"""

    tags = ["Quatrième"]
    function = _tex_proprietes


# ----PROPRIETES AVEC 10


def tex_proprietes_neg_val(exp_max, nb_max, typeexo):
    """
    Renvoie des valeurs pour l'exercice sur les propriétés des puissances
    @param exp_max: valeur maximale pour les exposants
    @type exp_max: Integer
    @param nb_max: Valeur maximale pour les nombres
    @type nb_max: integer
    @param typeexo: 0 : 2 exposants et 1 nombre ; 1 : 1 exposant et 2 nombres
    @type typeexo: integer
    """

    if typeexo:
        while 1:
            nb1 = randrange(2, nb_max)
            nb2 = randrange(2, nb_max)
            exp1 = randrange(2, exp_max)
            exp2 = exp1
            if nb1 != nb2:
                break
    else:
        while 1:
            nb1 = randrange(2, nb_max)
            nb2 = nb1
            exp1 = randrange(2, exp_max)
            exp2 = randrange(2, exp_max)
            if exp1 != exp2:
                break
    return (nb1, exp1, nb2, exp2)


def _tex_proprietes_neg():
    exo = ["\\exercice",
           u"Écrire sous la forme d'une puissance de 10 puis donner l'écriture",
           u" décimale de ces nombres :", "\\begin{multicols}{2}",
           "  \\noindent%", "  \\begin{enumerate}"]
    cor = ["\\exercice*",
           u"Écrire sous la forme d'une puissance de 10 puis donner l'écriture",
           u" décimale de ces nombres :", "\\begin{multicols}{2}",
           "  \\noindent%", "  \\begin{enumerate}"]
    lexos = [0, 1, 2, 3, 0, 1, 2, 3]

    # 0: a^n*a^p ; 1: (a^n)^p ; 2:a^n/a^p

    for dummy in range(len(lexos)):
        lexp = [randrange(-6, 6) for dummy in range(2)]
        j = lexos.pop(randrange(len(lexos)))

        # FIXME : À finir

        if j == 0:
            while abs(lexp[0] + lexp[1]) > 10:
                lexp = [randrange(-6, 6) for dummy in range(2)]
            exo.append("\\item $10^{%s} \\times 10^{%s} = \\dotfill$" % 
                       tuple(lexp))
            cor.append("\\item $10^{%s}\\times 10^{%s}=" % tuple(lexp))
            cor.append("10^{%s+%s}=" % (lexp[0], tex_coef(lexp[1],
                       '', bpn=1)))
            cor.append("10^{%s}=%s$" % (lexp[0] + lexp[1],
                       decimaux(10 ** (lexp[0] + lexp[1]), 1)))
        elif j == 1:
            while abs(lexp[0] * lexp[1]) > 10:
                lexp = [randrange(-6, 6) for dummy in range(2)]
            exo.append("\\item $(10^{%s})^{%s}=\\dotfill$" % (lexp[0],
                       lexp[1]))
            cor.append("\\item $(10^{%s})^{%s}=" % tuple(lexp))
            cor.append("10^{%s \\times %s}=" % (lexp[0], tex_coef(lexp[1],
                       '', bpn=1)))
            cor.append("10^{%s}=%s$" % (lexp[0] * lexp[1],
                       decimaux(10 ** (lexp[0] * lexp[1]), 1)))
        elif j == 2:
            while abs(lexp[0] - lexp[1]) > 10:
                lexp = [randrange(-6, 6) for dummy in range(2)]
            exo.append("\\item $\\dfrac{10^{%s}}{10^{%s}}=\\dotfill$" % 
                       tuple(lexp))
            cor.append("\\item $\\dfrac{10^{%s}}{10^{%s}}=" % tuple(lexp))
            cor.append("10^{%s-%s}=" % (lexp[0], tex_coef(lexp[1],
                       '', bpn=1)))
            cor.append("10^{%s}=%s$" % (lexp[0] - lexp[1],
                       decimaux(10 ** (lexp[0] - lexp[1]), 1)))
    exo.append("\\end{enumerate}")
    exo.append("\\end{multicols}\n")
    cor.append("\\end{enumerate}")
    cor.append("\\end{multicols}\n")
    return (exo, cor)

class tex_proprietes_neg(LegacyExercise):
    """Propriétés sur les puissances de 10"""

    tags = ["Quatrième"]
    function = _tex_proprietes_neg


#------------------------------------------------------------------------------
#  Écritures scientifiques
#------------------------------------------------------------------------------


def val_sc():
    while True:
        a = randrange(10) * 1000 + randrange(10) + randrange(10) * 10 ** \
            (randrange(2) + 1)
        a = a * 10 ** randrange(-9, 6)

        # a=randrange(1,9999)*10**randrange(-9,6)

        if a >= 10 or (a < 1 and a > 0) :
            break
    return a


def _ecr_sc():
    from math import log10, floor
    exo = ["\\exercice", u"Compléter par le nombre qui convient :",
           "\\begin{multicols}{3}", "  \\noindent%",
           "  \\begin{enumerate}"]
    cor = ["\\exercice*",
           u"Compléter par le nombre qui convient :",
           "\\begin{multicols}{3}", "  \\noindent%",
           "  \\begin{enumerate}"]
    for dummy in range(6):
        a = val_sc()
        exp = int(floor(log10(a)))
        a_sc = a / 10 ** exp
        s_a = decimaux(a, 1)
        s_a_sc = decimaux(a_sc, 1)
        if randrange(2):  # forme : a=a_sc*...
            exo.append("\\item $%s=%s\\times\\dotfill$" % (s_a,
                       s_a_sc))
            cor.append("\\item $%s=%s\\times\\mathbf{10^{%s}}$" % (s_a,
                       s_a_sc, decimaux(exp, 1)))
        else:
            # forme : a_sc*...=a
            exo.append("\\item $%s\\times\\dotfill=%s$" % (s_a_sc,
                       s_a))
            cor.append("\\item $%s\\times\\mathbf{10^{%s}}=%s$" % (s_a_sc,
                       decimaux(exp, 1), s_a))
    exo.append("\\end{enumerate}")
    exo.append("\\end{multicols}\n")
    cor.append("\\end{enumerate}")
    cor.append("\\end{multicols}\n")
    return (exo, cor)

class ecr_sc(LegacyExercise):
    """Écritures scientifiques"""

    tags = ["Quatrième"]
    function = _ecr_sc


# ------------------- PUISSANCES de 10 -------------------


def _exo_puissances():
    from math import floor, log10
    sd = str.maketrans('.', ',')  # convertit les . en , (separateur decimal)
    exo = ["\\exercice",
           u"Calculer les expressions suivantes et donner l'écriture scientifique du résultat.",
           "\\begin{multicols}{2}", "  \\noindent%"]
    cor = ["\\exercice*",
           u"Calculer les expressions suivantes et donner l'écriture scientifique du résultat.",
           "\\begin{multicols}{2}", "  \\noindent%"]
    valeurs = valeurs_puissances()
    i = randrange(2)
    exo.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_0(valeurs[i]).translate(sd))
    cor.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_0(valeurs[i]).translate(sd))
    cor.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_1(valeurs[i]).translate(sd))
    cor.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_2(valeurs[i]).translate(sd))
    if int(floor(log10(valeurs[i][0] * valeurs[i][1] / valeurs[i][2]))) !=  0:
        cor.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_3(valeurs[i]).translate(sd))
    cor.append("\\[ \\boxed{\\thenocalcul = %s} \\]" % tex_puissances_4(valeurs[i]).translate(sd))
    exo.append("\\columnbreak\\stepcounter{nocalcul}%")
    cor.append("\\columnbreak\\stepcounter{nocalcul}%")
    exo.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_0(valeurs[1 - i]).translate(sd))
    cor.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_0(valeurs[1 - i]).translate(sd))
    cor.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_1(valeurs[1 - i]).translate(sd))
    cor.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_2(valeurs[1 - i]).translate(sd))
    if int(floor(log10(valeurs[1 - i][0] * valeurs[1 - i][1] / valeurs[1 - i][2]))) != 0:
        cor.append("\\[ \\thenocalcul = %s \\]" % tex_puissances_3(valeurs[1 - i]).translate(sd))
        cor.append("\\[ \\boxed{\\thenocalcul = %s} \\]" % tex_puissances_4(valeurs[1 - i]).translate(sd))
    exo.append("\\end{multicols}\n")
    cor.append("\\end{multicols}\n")
    return (exo, cor)

class exo_puissances(LegacyExercise):
    """Puissances de 10"""

    tags = ["Quatrième"]
    function = _exo_puissances


def tex_puissances_0(a):
    if isinstance(a, tuple):
        return '\\cfrac{\\nombre{%s} \\times 10^{%s} \\times \\nombre{%s} \\times 10^{%s}}{\\nombre{%s} \\times \\big( 10^{%s} \\big) ^%s}' % \
            (decimaux(a[0]), decimaux(a[3]), decimaux(a[1]), decimaux(a[4]), decimaux(a[2]), decimaux(a[5]), decimaux(a[6]))


def tex_puissances_1(a):
    if isinstance(a, tuple):
        if a[4] < 0:
            return '\\cfrac{\\nombre{%s} \\times \\nombre{%s}}{\\nombre{%s}} \\times \\cfrac{10^{%s+(%s)}}{10^{%s \\times %s}}' % \
                (decimaux(a[0]), decimaux(a[1]), decimaux(a[2]), decimaux(a[3]), decimaux(a[4]), decimaux(a[5]), decimaux(a[6]))
        else:
            return '\\cfrac{\\nombre{%s} \\times \\nombre{%s}}{\\nombre{%s}} \\times \\cfrac{10^{%s+%s}}{10^{%s \\times %s}}' % \
                (decimaux(a[0]), decimaux(a[1]), decimaux(a[2]), decimaux(a[3]), decimaux(a[4]), decimaux(a[5]), decimaux(a[6]))


def tex_puissances_2(a):
    if isinstance(a, tuple):
        if a[0] * a[1] / a[2] == a[0] * a[1] // a[2]:
            if a[5] * a[6] < 0:
                return '\\nombre{%s} \\times 10^{%s-(%s)}' % \
                       (decimaux((a[0] * a[1]) // a[2]), decimaux(a[3] + a[4]), decimaux(a[5] * a[6]))
            else:
                return '\\nombre{%s} \\times 10^{%s-%s}' % (decimaux(a[0] * a[1] / a[2]), decimaux(a[3] + a[4]), decimaux(a[5] * a[6]))
        else:
            if a[5] * a[6] < 0:
                return '\\nombre{%s} \\times 10^{%s-(%s)}' % \
                       (decimaux(a[0] * a[1] / a[2]), decimaux(a[3] + a[4]), decimaux(a[5] * a[6]))
            else:
                return '\\nombre{%s} \\times 10^{%s-%s}' % (decimaux(a[0] * a[1] / a[2]), decimaux(a[3] + a[4]), decimaux(a[5] * a[6]))


def tex_puissances_3(a):
    from math import floor, log10
    b = int(floor(log10(a[0] * a[1] / a[2])))
    if isinstance(a, tuple) and b != 0:
        return '\\nombre{%s}  \\times 10^{%s} \\times 10^{%s}' % \
               (decimaux(a[0] * a[1] / a[2] / 10 ** b), decimaux(b), decimaux(a[3] + a[4] - a[5] * a[6]))


def tex_puissances_4(a):
    from math import floor, log10
    b = int(floor(log10(a[0] * a[1] / a[2])))
    if isinstance(a, tuple):
        return '\\nombre{%s}  \\times 10^{%s}' % (decimaux(a[0] * a[1] / a[2] / 10 ** b), decimaux(b + a[3] + a[4] - a[5] * a[6]))


def verifie_type(a):  # verifie si des nombres reels dans le tuple a sont en fait des nombres entiers et change leur type
    liste = []
    for i in range(len(a)):
        if str(a[i]).endswith('.0'):
            liste.append(int(a[i] + .1))
        else:
            liste.append(a[i])
    return tuple(liste)


def valeurs_puissances():  # renvoie un tuple contenant les valeurs pour les deux exercices sur les puissances
    from math import floor, log10
    (maxi, emax) = (10, 2)
    while True:
        (b1, b2) = (valeur_alea(2, maxi), valeur_alea(2, maxi))
        (b1, b2) = (b1 // pgcd(b1, b2), b2 // pgcd(b1, b2))
        if b1 != 1 and b2 != 1:
            break
    while True:
        (n1, n2) = ((b1 * valeur_alea(2, maxi)) * 10 ** randrange(-emax,
                    emax), (b2 * valeur_alea(2, maxi)) * 10 ** randrange(-emax,
                    emax))
        n3 = ((b1 * b2) * choice((2, 4, 5, 8))) * 10 ** randrange(-emax,
                emax)
        if int(floor(log10(n1 * n2 / n3))) != 0 and n1 != 1 and \
            n2 != 1 and n3 != 1:
            break
    (e1, e2, e3, e4) = (valeur_alea(-10, 10), valeur_alea(-10, 10),
                        valeur_alea(2, 10), valeur_alea(2, 5))
    a = (n1, n2, n3, e1, e2, e3, e4)
    while True:
        (b1, b2) = (valeur_alea(2, maxi), valeur_alea(2, maxi))
        (b1, b2) = (b1 // pgcd(b1, b2), b2 // pgcd(b1, b2))
        if b1 != 1 and b2 != 1:
            break
    (n1, n2) = ((b1 * valeur_alea(2, maxi)) * 10 ** randrange(-emax, emax + 
                1), (b2 * valeur_alea(2, maxi)) * 10 ** randrange(-emax,
                emax + 1))
    n3 = ((b1 * b2) * choice((1, 2, 4, 5, 8))) * 10 ** randrange(-emax,
            emax + 1)
    (e1, e2, e3, e4) = (valeur_alea(-10, 10), valeur_alea(-10, 10),
                        valeur_alea(-10, -2), valeur_alea(2, 5))
    b = (n1, n2, n3, e1, e2, e3, e4)
    return (a, b)



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import random
from math import atan, cos, pi, sin
from pyromaths.ex import LegacyExercise


# ===============================================================================
# Symétrique d'une figure par rapport à une droite avec quadrillage
# ===============================================================================


def valeurs_quad2(nb_pts):
    vals = []
    for i in range(nb_pts):
        angle = random.randrange(i * 360 // nb_pts, (i + 1) * 360 // nb_pts)
        vals.append((random.randrange(1, 7) * .5 * cos(angle * pi / 180),
                     random.randrange(1, 7) * .5 * sin(angle * pi / 180)))
    return vals


def valeurs_quad(nb_pts):
    vals = []
    for i in range(nb_pts):
        (alpha, beta) = ((i * 360) // nb_pts, ((i + 1) * 360) // nb_pts)
        (x, y, angle) = (0, 0, 0)
        while x == 0 or angle < alpha or angle > beta:
            (x, y) = (random.randrange(-6, 7) * .5, random.randrange(-6, 7) * .5)
            if x > 0:
                angle = int(atan(y / x) * 180 / pi + 360) % 360
            if x < 0:
                angle = int(atan(y / x) * 180 / pi + 180)
        vals.append((x, y))
    return vals


def place_pts(vals, angle):
    txt = ["  \\pstGeonode[PointSymbol=x,PointName=none]"]
    for i in range(len(vals)):
        txt.append("(%s,%s)" % vals[i])
        txt.append("{%s}" % chr(i + 97))
    txt.append("\n\\pstGeonode[PointSymbol=none,PointName=none](-4.5;%s){A}(4.5;%s){B}" % (angle, angle))
    txt.append("\n\\psline[linecolor=Black, linewidth=1pt, nodesep=-4.5](A)(B)")
    txt.append("\n\\pspolygon[linecolor=Maroon, linewidth=1pt]")
    for i in range(len(vals)):
        txt.append("(%s)" % chr(i + 97))
    return "".join(txt)


def _SymetrieQuadrillage():
    exo = ["\\exercice", u"Construire la symétrique de chacune des figures par rapport à la droite en",
           "utilisant le quadrillage :\\par", "\\psset{unit=.9cm}"]
    cor = ["\\exercice*", u"Construire la symétrique de chacune des figures par rapport à la droite en",
           "utilisant le quadrillage :\\par", "\\psset{unit=.9cm}"]

    nbpts = 5
    langles = [0, 90, 45, 135]
    for j in range(3):
        angle = langles.pop(random.randrange(len(langles)))
        vals = valeurs_quad(nbpts)
        txt = place_pts(vals, angle)
        exo.append("\\begin{pspicture*}(-3,-3)(3,3)")
        exo.append("\\psgrid[subgriddiv=2,gridlabels=0pt]")
        exo.append(txt)
        cor.append("\\begin{pspicture*}(-3,-3)(3,3)")
        cor.append("\\psgrid[subgriddiv=2,gridlabels=0pt]")
        cor.append(txt)
        txt_cor = "\\pstOrtSym[PointSymbol=x,PointName=none]{A}{B}{"
        for i in range(len(vals)):
            if i > 0:
                txt_cor += ","
            txt_cor += "%s" % chr(i + 97)
        txt_cor += "}["
        for i in range(len(vals)):
            if i > 0:
                txt_cor += ","
            txt_cor += "%s1" % chr(i + 97)
        txt_cor += "]\n  \pspolygon[linecolor=Black, linestyle=dashed, linewidth=1pt]"
        for i in range(len(vals)):
            txt_cor += "(%s1)" % chr(i + 97)
        cor.append(txt_cor)
        exo.append("\\end{pspicture*}")
        cor.append("\\end{pspicture*}")
        if j < 2:
            exo.append("\\hfill")
            cor.append("\\hfill")
    return exo, cor


class SymetrieQuadrillage(LegacyExercise):
    """Symétrie et quadrillages"""

    tags = ["Sixième"]
    function = _SymetrieQuadrillage



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from decimal import Decimal
import functools
import random
import textwrap

from pyromaths import ex
from pyromaths.outils.Arithmetique import pgcd

FRANCAIS_ORDINAL = {
    1: u"premier",
    2: u"deuxième",
    3: u"troisième",
    4: u"quatrième",
    5: u"cinquième",
    6: u"sixième",
    7: u"septième",
    }
FRANCAIS_INVERSE = {
    2: u"à la moitié du",
    3: u"au tiers du",
    4: u"au quart du",
    5: u"au cinquième du",
    6: u"au sixième du",
    7: u"au septième du",
    8: u"au huitième du",
    9: u"au neuvième du",
    10: u"au dixième du",
    }
FRANCAIS_MULTIPLE = {
    2: u"au double du",
    3: u"au triple du",
    4: u"au quadruple du",
    5: u"à cinq fois le",
    6: u"à six fois le",
    7: u"à sept fois le",
    8: u"à huit fois le",
    9: u"à neuf fois le",
    10: u"à dix fois le",
    }
NOTATIONS = [
    "u",
    r"\left(u_n\right)",
    r"\left(u_n\right)_{n\in\mathbb{N}}",
    ]

def signe(nombre):
    if nombre > 0:
        return 1
    if nombre < 0:
        return -1
    raise ValueError

class Fraction:
    def __init__(self, numerateur, denominateur, signe=1):
        self.signe = signe
        self.numerateur = numerateur
        self.denominateur = denominateur

    def latex(self):
        if self.denominateur == 1:
            return str(self.signe * self.numerateur)
        else:
            if self.signe == 1:
                signe = ""
            else:
                signe = "-"
            return r"{}\frac{{ {} }}{{ {} }}".format(
                    signe,
                    self.numerateur,
                    self.denominateur,
                    )

    def simplifie(self):
        if self.numerateur % self.denominateur == 0:
            return Entier(self.signe * Decimal(self.numerateur) / Decimal(self.denominateur))
        diviseur = pgcd(self.numerateur, self.denominateur)
        return Fraction(
                self.numerateur // diviseur,
                self.denominateur // diviseur,
                self.signe,
                )

    def __float__(self):
        return float(self.signe * self.numerateur / self.denominateur)



@functools.total_ordering
class Entier:
    def __init__(self, valeur):
        self.valeur = valeur

    def __int__(self):
        return self.valeur

    def __neg__(self):
        return self.__class__(- self.valeur)

    def __lt__(self, other):
        if isinstance(other, int):
            return self.valeur < other
        elif isinstance(other, Entier):
            return self.valeur < other.valeur
        else:
            raise TypeError()

    def __eq__(self, other):
        if isinstance(other, int):
            return self.valeur == other

        elif isinstance(other, Entier):
            return self.valeur == other.valeur
        else:
            raise TypeError()

    def latex(self, signe="-"):
        if signe == "+":
            if self.valeur >= 0:
                return "+" + str(self.valeur)
            else:
                return "-" + str(abs(self.valeur))
        else:
            if self.valeur >= 0:
                return str(self.valeur)
            else:
                return "-" + str(abs(self.valeur))

    def __float__(self):
        return float(self.valeur)

FRACTIONS = [
        Fraction(2, 1),
        Fraction(3, 1),
        Fraction(4, 1),
        Fraction(5, 1),
        Fraction(10, 1),
        Fraction(1, 2),
        Fraction(1, 3),
        Fraction(1, 4),
        Fraction(1, 5),
        Fraction(1, 10),
        Fraction(2, 3),
        Fraction(1, 4),
        Fraction(3, 4),
        Fraction(2, 5),
        Fraction(3, 5),
        Fraction(4, 5),
        ]

class Fonction:

    def calcul(self, argument):
        raise NotImplementedError()

    def resultat(self, variable):
        raise NotImplementedError()

    def expression(self, variable):
        raise NotImplementedError()

class Lineaire(Fonction):

    def __init__(self):
        self.coeff = random.choice(FRACTIONS)

    def expression(self, variable):
        return r"{coeff}{variable}".format(
            coeff=self.coeff.latex(),
            variable=variable,
            )

    def calcul(self, argument):
        if isinstance(argument, Entier):
            if float(argument) < 0:
                arg = r"\left( {} \right)".format(argument.latex())
            else:
                arg = argument.latex()
            yield self.expression(r"\times " + arg)
            if isinstance(self.coeff.simplifie(), Fraction):
                yield Fraction(
                            self.coeff.numerateur * argument.valeur,
                            self.coeff.denominateur,
                            ).latex()
                if pgcd(self.coeff.numerateur * argument.valeur, self.coeff.denominateur) != 1:
                    yield self.resultat(argument).latex()
            else:
                yield self.resultat(argument).latex()
        else:
            yield self.expression(r"\times " + argument.latex())
            yield Fraction(
                self.coeff.numerateur * argument.numerateur,
                self.coeff.denominateur * argument.denominateur,
                ).latex()
            if pgcd(
                self.coeff.numerateur * argument.numerateur,
                self.coeff.denominateur * argument.denominateur,
                ) != 1:
                yield self.resultat(argument).latex()

    def resultat(self, variable):
        if isinstance(variable, Entier):
            variable = Fraction(variable.valeur, 1)
        return Fraction(
            self.coeff.numerateur * variable.numerateur,
            self.coeff.denominateur * variable.denominateur,
            ).simplifie()

class Affine(Fonction):

    def __init__(self):
        self.coeff = random.choice(FRACTIONS)
        ordonnee = random.randint(1, 10)
        if random.randint(1, 2) == 1:
            ordonnee = -ordonnee
        self.ordonnee = Entier(ordonnee)

    def expression(self, variable):
        return r"{coeff}{variable}{ordonnee}".format(
            coeff=self.coeff.latex(),
            ordonnee=self.ordonnee.latex("+"),
            variable=variable,
            )

    def calcul(self, argument):
        if float(argument) < 0:
            arg = r"\left( {} \right)".format(argument.latex())
        else:
            arg = argument.latex()
        yield self.expression(r"\times " + arg)
        if isinstance(argument, Entier):
            if isinstance(self.coeff.simplifie(), Fraction):
                yield r"{fraction} {signe} \frac{{ {ordonnee} \times {denom} }}{{ {denom} }}".format(
                        fraction=Fraction(
                            self.coeff.numerateur * argument.valeur,
                            self.coeff.denominateur,
                            ).latex(),
                        ordonnee=abs(self.ordonnee.valeur),
                        denom=self.coeff.denominateur,
                        signe=Entier(self.coeff.denominateur * self.ordonnee.valeur).latex("+")[0],
                        )
                yield r"\frac{{ {gauche} {droite} }}{{ {denom} }}".format(
                        gauche=self.coeff.numerateur * argument.valeur,
                        droite=Entier(self.coeff.denominateur * self.ordonnee.valeur).latex("+"),
                        denom=self.coeff.denominateur,
                        )
        else:
            yield r"{} + \frac{{ {} \times {} }}{{ {} }}".format(
                Fraction(
                    self.coeff.numerateur * argument.numerateur,
                    self.coeff.denominateur * argument.denominateur,
                    ).latex(),
                self.ordonnee.valeur,
                self.coeff.denominateur * argument.denominateur,
                self.coeff.denominateur * argument.denominateur,
                )
            yield r"\frac{{ {} {} }}{{ {} }}".format(
                self.coeff.numerateur * argument.numerateur,
                Entier(self.ordonnee.valeur * self.coeff.denominateur * argument.denominateur).latex("+"),
                self.coeff.denominateur * argument.denominateur,
                )
        yield self.resultat(argument).latex()

    def resultat(self, variable):
        if isinstance(variable, Entier):
            variable = Fraction(variable.valeur, 1)
        return Fraction(
            Decimal(self.coeff.numerateur * variable.numerateur + self.ordonnee.valeur * self.coeff.denominateur * variable.denominateur),
            Decimal(self.coeff.denominateur * variable.denominateur),
            ).simplifie()

class FractionProduit(Fonction):

    def __init__(self):
        self.numerateur = Entier(random.randint(2, 10))
        self.denominateur = Entier(random.randint(2, 10))

    def expression(self, variable):
        return r"\frac{{ {numerateur}^{variable} }}{{ {denominateur}{variable} }}".format(
                numerateur=self.numerateur.latex("-"),
                denominateur=self.denominateur.latex("-"),
                variable=variable,
            )

    def calcul(self, argument):
        if argument.valeur < 0:
            arg2 = r"\left( {} \right)".format(argument.latex())
        else:
            arg2 = argument.latex()
        yield r"\frac{{ {numerateur}^{{ {argument} }}}}{{ {denominateur}\times{arg2} }}".format(
                numerateur=self.numerateur.latex(),
                denominateur=self.denominateur.latex(),
                argument=argument.latex(),
                arg2=arg2,
            )
        yield r"\frac{{ {} }}{{ {} }}".format(
                self.numerateur.valeur ** argument.valeur,
                self.denominateur.valeur * argument.valeur,
                )
        if pgcd(
                self.numerateur.valeur ** argument.valeur,
                self.denominateur.valeur * argument.valeur,
                ) != 1:
            yield self.resultat(argument).latex()

    def resultat(self, argument):
        return Fraction(
                self.numerateur.valeur ** argument.valeur,
                self.denominateur.valeur * argument.valeur,
                ).simplifie()

class Trinome(Fonction):

    def __init__(self):
        coef = [0]*3
        for i in [0, 1, 2]:
            coef[i] = random.randint(2, 5)
            if random.randint(0,1) == 0:
                coef[i] = -coef[i]
        self.coef = [Entier(i) for i in coef]

    def expression(self, variable):
        return r"{coef[2]}{variable}^2{coef[1]}{variable}{coef[0]}".format(
            coef=[self.coef[0].latex("+"), self.coef[1].latex("+"), self.coef[2].latex("-")],
            variable=variable,
            )

    def calcul(self, argument):
        if argument < 0:
            texargument = r"\left{{ {variable} }}".format(argument.latex())
        else:
            texargument = argument.latex()
        yield r"{coef[2]}\times{argument}^2{coef[1]}\times{argument}{coef[0]}".format(
            coef=[self.coef[0].latex("+"), self.coef[1].latex("+"), self.coef[2].latex("-")],
            argument=texargument,
            )
        yield r"{}{}{}".format(
            Entier(self.coef[2].valeur * argument.valeur**2).latex(),
            Entier(self.coef[1].valeur * argument.valeur).latex("+"),
            Entier(self.coef[0].valeur).latex("+")
            )
        yield Entier(
            (self.coef[2].valeur * argument.valeur**2) +
            (self.coef[1].valeur * argument.valeur) +
            (self.coef[0].valeur)
            ).latex()

    def resultat(self, argument):
        return Entier(self.coef[2].valeur * argument.valeur**2 + self.coef[1].valeur * argument.valeur + self.coef[0].valeur)


class IdentiteTranslatee(Fonction):

    def __init__(self):
        ordonnee = random.randint(1, 10)
        if random.randint(1, 2) == 1:
            ordonnee = -ordonnee
        self.ordonnee = Entier(ordonnee)

    def expression(self, variable):
        return r"{variable}{ordonnee}".format(
            variable=variable,
            ordonnee=self.ordonnee.latex("+"),
            )

    def calcul(self, argument):
        yield self.expression(argument.latex())
        yield self.resultat(argument).latex()

    def resultat(self, variable):
        return Entier(variable.valeur + self.ordonnee.valeur)

class FrancaisGeometrique(Fonction):

    def __init__(self):
        if random.randint(1, 2) == 1:
            self.raison = Fraction(1, random.randint(2, 10))
        else:
            self.raison = Entier(random.randint(2, 10))

    @property
    def francais(self):
        if isinstance(self.raison, Fraction):
            return u"{} précédent".format(FRANCAIS_INVERSE[self.raison.denominateur])
        else:
            return u"{} précédent".format(FRANCAIS_MULTIPLE[self.raison.valeur])

    def expression(self, variable):
        # L'argument est du code LaTeX
        return r"{} {}".format(self.raison.latex(), variable)

    def calcul(self, argument):
        yield r"{} \times {}".format(self.raison.latex(), argument.latex())
        resultat = self.resultat(argument).latex()
        if isinstance(resultat, Entier):
            yield self.resultat(argument).latex()
        else:
            if isinstance(argument, Entier):
                argument = Fraction(argument.valeur, 1)
            raison = self.raison
            if isinstance(raison, Entier):
                raison = Fraction(raison.valeur, 1)
            numerateur = argument.numerateur * raison.numerateur
            denominateur = raison.denominateur * argument.denominateur
            if denominateur != 1:
                yield r"\frac{{ {} }}{{ {} }}".format(numerateur, denominateur)
            if numerateur % denominateur == 0:
                yield Entier(numerateur //  denominateur).latex()
                return

            diviseur = pgcd(numerateur, denominateur)
            if diviseur == 1:
                return

            yield Fraction(numerateur // diviseur, denominateur // diviseur).latex()

    def resultat(self, argument):
        if isinstance(argument, Entier) and isinstance(self.raison, Entier):
            return Entier(argument.valeur * self.raison.valeur)
        else:
            if isinstance(argument, Entier):
                argument = Fraction(argument.valeur, 1)
            if isinstance(self.raison, Entier):
                raison = Fraction(self.raison.valeur, 1)
            else:
                raison = self.raison
            numerateur = argument.numerateur * raison.numerateur
            denominateur = raison.denominateur * argument.denominateur
            if numerateur % denominateur == 0:
                return Entier(numerateur //  denominateur)
            diviseur = pgcd(numerateur, denominateur)
            return Fraction(numerateur // diviseur, denominateur // diviseur)

class FrancaisArithmetique(Fonction):

    def __init__(self):
        self.raison = random.randint(1, 10)
        if random.randint(1, 2) == 1:
            self.raison = -self.raison

    @property
    def francais(self):
        if self.raison > 0:
            return u"au terme précédent auquel on ajoute {}".format(self.raison)
        else:
            return u"au terme précédent auquel on soustrait {}".format(-self.raison)

    def expression(self, variable):
        # L'argument est du code LaTeX
        return r"{} {}".format(variable, Entier(self.raison).latex("+"))

    def calcul(self, argument):
        yield self.expression(argument.latex())
        yield self.resultat(argument).latex()

    def resultat(self, argument):
        return Entier(argument.valeur + self.raison)

class FrancaisOppose(Fonction):

    @property
    def francais(self):
        return u"à l'opposé du précédent"

    def expression(self, variable):
        # L'argument est du code LaTeX
        if variable.startswith("-"):
            return r"-\left({}\right)".format(variable)
        else:
            return r"-{}".format(variable)

    def calcul(self, argument):
        yield Entier(-argument.valeur).latex("-")

    def resultat(self, argument):
        if not isinstance(argument, Entier):
            raise TypeError("Argument must be an instance of `Entier`.")
        return - argument

class FrancaisInverse(Fonction):

    @property
    def francais(self):
        return u"à l'inverse du précédent"

    def expression(self, variable):
        # L'argument est du code LaTeX
        return r"\frac{{1}}{{ {} }}".format(variable)

    def calcul(self, argument):
        yield self.expression(argument.latex())
        if isinstance(argument, Entier) and argument == 1:
            yield "1"
        elif isinstance(argument, Entier) and argument == -1:
            yield "-1"
        elif isinstance(argument, Entier) and argument > 0:
            pass
        elif isinstance(argument, Entier) and argument < 0:
            yield r"-{}".format(self.expression((-argument).latex()))
        elif isinstance(argument, Fraction):
            yield self.resultat(argument).latex()
        else:
            raise TypeError("Argument must be an instance of `Entier` or `Fraction`.")

    def resultat(self, argument):
        if isinstance(argument, Fraction):
            if argument.numerateur == 1:
                return Entier(argument.denominateur * argument.signe)
            else:
                return Fraction(
                    abs(argument.denominateur),
                    abs(argument.numerateur),
                    signe(argument.numerateur * argument.denominateur),
                    )
        elif isinstance(argument, Entier):
            return Fraction(1, abs(argument.valeur), signe(argument.valeur))
        raise TypeError("Argument must be an instance of `Entier` or `Fraction`.")


class Question:

    def __init__(self, indice0):
        self.indice0 = indice0
        if self.indice0 == 0:
            self.notation = NOTATIONS[random.randint(0, 2)]
        else:
            self.notation = NOTATIONS[random.randint(0, 1)]

class Francais(Question):

    def __init__(self, indice0):
        super().__init__(indice0)
        self.terme0 = Entier(random.randint(1, 10)*(-1)**random.randrange(2))
        self.fonction = random.choice([
            FrancaisGeometrique,
            FrancaisArithmetique,
            FrancaisOppose,
            FrancaisInverse,
            ])()

    @property
    def latex_params(self):
        return {
            'indice0': self.indice0,
            'terme0': self.terme0.latex("-"),
            'suivant': self.fonction.francais,
            'notation': self.notation,
            }

class General(Question):

    def __init__(self, indice0):
        super().__init__(indice0)
        self.fonction = random.choice([
            FractionProduit,
            Trinome,
            Affine,
            Lineaire,
            IdentiteTranslatee,
            ])()

    @property
    def latex_params(self):
        return {
            'indice0': self.indice0,
            'fonction': self.fonction.expression(r"n"),
            'notation': self.notation,
            }

class Recursif(Question):

    def __init__(self, indice0):
        super().__init__(indice0)
        self.terme0 = Entier(random.randint(-10, 10))
        self.fonction = random.choice([
            Affine,
            IdentiteTranslatee,
            Lineaire,
            ])()

    @property
    def latex_params(self):
        return {
            'indice0': self.indice0,
            'terme0': self.terme0.latex("-"),
            'fonction': self.fonction.expression(r"u_n"),
            'notation': self.notation,
            }

class TermesDUneSuite(ex.TexExercise):
    """Termes d'une suite"""

    tags = ["1re Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # * `self.rang[0]` désigne l'ordinal du premier terme demandé (pour la
        #   première question de chacune des trois suites) ;
        # * `self.rang[1]` et `self.rang[2]` sont les rangs demandés pour les
        #   deux questions suivantes dans chacune des trois suites.
        self.rang = [random.randint(2, 7)] + random.sample(list(range(3, 7)), 2)

        self.questions = [
                Francais(random.randint(0, min(self.rang[1:])-1)),
                General(random.randint(0, min(self.rang[1:])-1)),
                Recursif(random.randint(0, min(self.rang[1:])-1)),
                ]

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(r'Pour chacune des suites $u$ suivantes, calculer :')
        exo.append(r' (a) le {} terme ;'.format(FRANCAIS_ORDINAL[self.rang[0]]))
        exo.append(r' (b) le terme de rang {} ;'.format(self.rang[1]))
        exo.append(r' (c) $u_{}$.'.format(self.rang[2]))

        exo.append(r'\begin{enumerate}')
        exo.append(r'  \item ${notation}$ est une suite de premier terme $u_{indice0}={terme0}$, et dont chaque terme (sauf le premier) est égal {suivant}.'.format(**self.questions[0].latex_params))
        exo.append(r'  \item ${notation}$ est la suite définie pour $n\geqslant {indice0}$ par : $u_n={fonction}$.'.format(**self.questions[1].latex_params))
        exo.append(textwrap.dedent(r"""
            \item ${notation}$ est la suite définie pour $n\geqslant {indice0}$ par :
                \[\left\{{\begin{{array}}{{l}}
                  u_{indice0}={terme0}\\
                  \text{{Pour tout $n\geqslant {indice0}$ : }} u_{{n+1}}={fonction}.
              \end{{array}}\right.\]
              """).format(**self.questions[2].latex_params))
        exo.append(r'\end{enumerate}')
        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.append(r'\begin{enumerate}')
        # Question 0
        exo.append(r"  \item Selon l'énoncé, le premier terme de ${notation}$ est $u_{indice0}={terme0}$. Puisque chaque terme (sauf le premier) est égal {suivant}, on a :".format(**self.questions[0].latex_params))
        termes = dict([(self.questions[0].indice0, self.questions[0].terme0)])
        calcul_termes = []
        for indice in range(self.questions[0].indice0, max(self.rang[0] + self.questions[0].indice0 - 1, self.rang[1], self.rang[2])):
            calcul = r"$u_{indice}={fonction}".format(
                indice=indice+1,
                fonction=self.questions[0].fonction.expression("u_{}".format(indice)),
                )
            for etape in self.questions[0].fonction.calcul(termes[indice]):
                calcul += " =" + etape
            calcul += "$"
            termes[indice+1] = self.questions[0].fonction.resultat(termes[indice])
            calcul_termes.append(calcul)
        exo.append(" ; ".join(calcul_termes) + ".")
        exo.append(r'\begin{enumerate}')
        exo.append(r' \item Calcul du {} terme :'.format(FRANCAIS_ORDINAL[self.rang[0]]))
        enumeration = []
        for indice in range(0, self.rang[0]):
            enumeration.append(u"le {ordinal} terme est $u_{indice}$".format(ordinal=FRANCAIS_ORDINAL[indice+1], indice=self.questions[0].indice0+indice))
        exo.append(" ; ".join(enumeration) + ".")
        exo.append(r"Le terme demandé est donc : $u_{}={}$.".format(self.rang[0] + self.questions[0].indice0 - 1, termes[self.rang[0] + self.questions[0].indice0 - 1].latex()))
        exo.append(r'\item Le terme de rang {indice} est : $u_{indice}={valeur}$.'.format(indice=self.rang[1], valeur=termes[self.rang[1]].latex()))
        exo.append(r'\item Nous avons calculé que : $u_{indice}={valeur}$.'.format(indice=self.rang[2], valeur=termes[self.rang[2]].latex()))
        exo.append(r'\end{enumerate}')

        # Question 1
        exo.append(r'  \item La suite ${notation}$ est définie pour $n\geqslant {indice0}$ par : $u_n={fonction}$.'.format(**self.questions[1].latex_params))
        exo.append(r"Elle est donc définie par son terme général : pour calculer un terme de rang $n$, on peut calculer directement l'image de $n$ par la suite.")
        exo.append(r'\begin{enumerate}')
        exo.append(r' \item Calcul du {} terme :'.format(FRANCAIS_ORDINAL[self.rang[0]]))
        enumeration = []
        for indice in range(0, self.rang[0]):
            enumeration.append(u"le {ordinal} terme est $u_{indice}$".format(ordinal=FRANCAIS_ORDINAL[indice+1], indice=self.questions[1].indice0+indice))
        exo.append(" ; ".join(enumeration) + ".")
        exo.append(r"Le terme demandé est donc : $u_{}=".format(self.rang[0] + self.questions[1].indice0 - 1))
        calcul = []
        for etape in self.questions[1].fonction.calcul(Entier(self.rang[0] + self.questions[1].indice0 - 1)):
            calcul.append(etape)
        exo.append(u" = ".join(calcul) + r"$.")
        exo.append(r"La solution est $u_{{ {} }}={}$.".format(self.rang[0] + self.questions[1].indice0 - 1, self.questions[1].fonction.resultat(Entier(self.rang[0] + self.questions[1].indice0 - 1)).latex()))
        exo.append(r"\item Le terme de rang {rang} est $u_{{ {rang} }}$.".format(rang=self.rang[1]))
        if self.rang[0] + self.questions[1].indice0 - 1 == self.rang[1]:
            exo.append(r"Ce terme a déjà été calculé, et $u_{{ {} }}={}$.".format(self.rang[1], self.questions[1].fonction.resultat(Entier(self.rang[1])).latex()))
        else:
            calcul = []
            for etape in self.questions[1].fonction.calcul(Entier(self.rang[1])):
                calcul.append(etape)
            exo.append(r"Le terme demandé est donc : $u_{{ {} }}=".format(self.rang[1]) + " = ".join(calcul) + r"$.")
            exo.append(r"La solution est donc  : $u_{{ {} }}={}$.".format(self.rang[1], self.questions[1].fonction.resultat(Entier(self.rang[1])).latex()))
        exo.append(r"\item")
        if self.rang[0] + self.questions[1].indice0 - 1 == self.rang[2]:
            exo.append(r"Ce terme a déjà été calculé, et $u_{{ {} }}={}$.".format(self.rang[2], self.questions[1].fonction.resultat(Entier(self.rang[2])).latex()))
        else:
            calcul = []
            for etape in self.questions[1].fonction.calcul(Entier(self.rang[2])):
                calcul.append(etape)
            exo.append(r"On a : $u_{{ {} }}=".format(self.rang[2]) + " = ".join(calcul) + r"$.")
            exo.append(r"La solution est donc  : $u_{{ {} }}={}$.".format(self.rang[2], self.questions[1].fonction.resultat(Entier(self.rang[2])).latex()))
        exo.append(r'\end{enumerate}')

        # Question 2
        exo.append(textwrap.dedent(r"""
            \item La suite ${notation}$ est définie par récurrence, pour $n\geqslant {indice0}$, par :
                \[\left\{{\begin{{array}}{{l}}
                  u_{indice0}={terme0}\\
                  \text{{Pour tout $n\geqslant {indice0}$ : }} u_{{n+1}}={fonction}.
              \end{{array}}\right.\]
              """).format(**self.questions[2].latex_params))
        termes = dict([(self.questions[2].indice0, self.questions[2].terme0)])
        calcul_termes = []
        for indice in range(self.questions[2].indice0, max(self.rang[0] + self.questions[2].indice0 - 1, self.rang[1], self.rang[2])):
            calcul = r"u_{indice} &= {fonction}".format(
                indice=indice+1,
                fonction=self.questions[2].fonction.expression("u_{}".format(indice)),
                )
            for etape in self.questions[2].fonction.calcul(termes[indice]):
                calcul += " =" + etape
            termes[indice+1] = self.questions[2].fonction.resultat(termes[indice])
            calcul_termes.append(calcul)
        exo.append(r"\begin{align*}")
        exo.append(r" \\".join(calcul_termes))
        exo.append(r"\end{align*}")
        exo.append(r'\begin{enumerate}')
        exo.append(r' \item Calcul du {} terme :'.format(FRANCAIS_ORDINAL[self.rang[0]]))
        enumeration = []
        for indice in range(0, self.rang[0]):
            enumeration.append(u"le {ordinal} terme est $u_{indice}$".format(ordinal=FRANCAIS_ORDINAL[indice+1], indice=self.questions[2].indice0+indice))
        exo.append(" ; ".join(enumeration) + ".")
        exo.append(r"Le terme demandé est donc : $u_{}={}$.".format(self.rang[0] + self.questions[2].indice0 - 1, termes[self.rang[0] + self.questions[2].indice0 - 1].latex()))
        exo.append(r'\item Le terme de rang {indice} est : $u_{indice}={valeur}$.'.format(indice=self.rang[1], valeur=termes[self.rang[1]].latex()))
        exo.append(r'\item Nous avons calculé que : $u_{indice}={valeur}$.'.format(indice=self.rang[2], valeur=termes[self.rang[2]].latex()))
        exo.append(r'\end{enumerate}')

        exo.append(r'\end{enumerate}')
        return "\n".join(exo)



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collége ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jéréme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from pyromaths.outils import Arithmetique
from pyromaths.outils.Priorites3 import texify, priorites
from pyromaths.ex import LegacyExercise
import random
# from pyromaths.classes.Fractions import Fraction  # Classe Fractions de pyromaths

def valeurs_somme_positive():
    """Travail sur les sommes de fractions en Quatrième"""
    l = []

    op = "+-"[random.randrange(2)]
    n2, d2 = random.randrange(1, 11), random.randrange(2, 11)
    if op == "-" and 1 - n2 //d2 > 0:
        l.append('1 %s Fraction(%s, %s)' % (op, n2, d2))
    else:
        l.append('Fraction(%s, %s) %s 1' % (n2, d2, op))

    op = "+-"[random.randrange(2)]
    n1 = random.randrange(2, 11)
    n2, d2 = random.randrange(1, 11), random.randrange(2, 11)
    if op == "-" and n1 - n2 //d2 > 0:
        l.append('%s %s Fraction(%s, %s)' % (n1, op, n2, d2))
    else:
        l.append('Fraction(%s, %s) %s %s' % (n2, d2, op, n1))

    op = "+-"[random.randrange(2)]
    n1 = random.randrange(1, 9) + random.randrange(1, 9) / 10
    n2, d2 = random.randrange(1, 11), random.randrange(2, 11)
    if op == "-" and n1 - n2 // d2 > 0:
        l.append('%s %s Fraction(%s, %s)' % (n1, op, n2, d2))
    else:
        l.append('Fraction(%s, %s) %s %s' % (n2, d2, op, n1))

    op = "+-"[random.randrange(2)]
    n1 = random.randrange(1, 11)
    n2, d2 = random.randrange(1, 11), random.randrange(2, 11)
    if op == "-" and n1 - n2 > 0:
        l.append('Fraction(%s, %s) %s Fraction(%s, %s)' % (n1, d2, op, n2, d2))
    else:
        l.append('Fraction(%s, %s) %s Fraction(%s, %s)' % (n2, d2, op, n1, d2))

    op = "+-"[random.randrange(2)]
    n1, d1 = random.randrange(1, 11), random.randrange(2, 11)
    n2, d2 = random.randrange(1, 11), random.randrange(2, 11) * d1
    if op == "-" and n1 // d1 - n2 // d2 > 0:
        l.append('Fraction(%s, %s) %s Fraction(%s, %s)' % (n1, d1, op, n2, d2))
    else:
        l.append('Fraction(%s, %s) %s Fraction(%s, %s)' % (n2, d2, op, n1, d1))

    for dummy in range(3):
        op = "+-"[random.randrange(2)]
        d1, d2 = 2, 2
        lepgcd = Arithmetique.pgcd(d1, d2)
        while lepgcd == d1 or lepgcd == d2:
            n1, d1 = random.randrange(1, 11), random.randrange(2, 11)
            n2, d2 = random.randrange(1, 11), random.randrange(2, 11)
            lepgcd = Arithmetique.pgcd(d1, d2)
        if op == "-" and n1 // d1 - n2 // d2 > 0:
            l.append('Fraction(%s, %s) %s Fraction(%s, %s)' % (n1, d1, op, n2, d2))
        else:
            l.append('Fraction(%s, %s) %s Fraction(%s, %s)' % (n2, d2, op, n1, d1))

    random.shuffle(l)
    return l

def _sommes_fractions_positives():
    exo = ["\\exercice",
           u"Calculer en détaillant les étapes. Donner le résultat sous la forme d'une fraction la plus simple possible (ou d'un entier lorsque c'est possible).",
           "\\begin{multicols}{4}", "  \\begin{enumerate}"]
    cor = ["\\exercice*",
           u"Calculer en détaillant les étapes. Donner le résultat sous la forme d'une fraction la plus simple possible (ou d'un entier lorsque c'est possible).",
           "\\begin{multicols}{4}", "  \\begin{enumerate}"]
    lexo = valeurs_somme_positive()
    for question in lexo:
        solve = [question]
        exo.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        cor.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        solve = priorites(question)
        solve = texify(solve)
        for e in solve:
            cor.append("\\[\\thenocalcul = " + e + "\\]")
        exo.append("\\stepcounter{nocalcul}")
        cor.append("\\stepcounter{nocalcul}")
    exo.extend(["  \\end{enumerate}", "\\end{multicols}"])
    cor.extend(["  \\end{enumerate}", "\\end{multicols}"])
    return (exo, cor)

class sommes_fractions_positives(LegacyExercise):
    """Sommes de positifs en écriture fractionnaire"""

    tags = ["Quatrième"]
    function = _sommes_fractions_positives

#===============================================================================
# def sommes_fractions_4e(op, level):
#     '''Choisit des valeurs aléatoires pour effectuer une somme ou une différence
#     de fractions en fonction du niveau de difficulté souhaité (de 1 é 4) et renvoie
#     l'énoncé et le corrigé au format TeX
#
#     @param op: '+' ou '-'
#     @param level: niveau de difficulté :
#                   1- Fractions positives et dénominateur de l'une multiple de l'autre
#                   2- Fractions positives et dénominateurs non multiples
#                   3- Fractions avec des nombres relatifs
#                   4- Fractions avec des nombres relatifs et résultats simplifiable
#     '''
#
#     while True:
#         (n1, d1, n2, d2) = (2, 2, 2, 2)
# #        import pdb; pdb.set_trace()
#         while True:
#             if level == 1:
#                 n1 = random.randrange(1, 16)
#                 d1 = random.randrange(1, 9)
#                 n2 = random.randrange(1, 16)
#                 d2 = d1*random.randrange(2, 11)
#                 if random.randrange(2):
#                     d1, d2 = d2, d1
#             elif level == 2:
#                 n1 = Arithmetique.valeur_alea(1, 16)
#                 d1 = Arithmetique.valeur_alea(1, 40)
#                 n2 = Arithmetique.valeur_alea(1, 16)
#                 d2 = Arithmetique.valeur_alea(1, 40)
#             else:
#                 while True:
#                     neg=[(-1,1)[random.randrange(2)] for x in range(4)]
#                     if neg[0]<0 or neg[1]<0 or neg[2]<0 or neg[3]<0:
#                         break
#                 n1 = random.randrange(1, 16)*neg[0]
#                 d1 = random.randrange(1, 40)*neg[1]
#                 n2 = random.randrange(1, 16)*neg[2]
#                 d2 = random.randrange(1, 40)*neg[3]
#             fr1 = Fractions(n1, d1)
#             fr2 = Fractions(n2, d2)
#             if Arithmetique.pgcd(fr1.n, fr1.d) == 1 and \
#                 Arithmetique.pgcd(fr2.n, fr2.d) == 1 and \
#                 (level == 1 or (Arithmetique.pgcd(fr1.d, fr2.d) != abs(fr1.d) \
#                                 and Arithmetique.pgcd(fr1.d, fr2.d) != abs(fr2.d))):
#                 if op == "+":
#                     simplifiable = abs(fr1.d * fr2.d) != \
#                                         abs(Fractions.simplifie(fr1 + fr2).d)
#                 else:
#                     simplifiable = abs(fr1.d * fr2.d) != \
#                                         abs(Fractions.simplifie(fr1 - fr2).d)
#                 if level == 1 or (simplifiable and level == 4) or \
#                                             (not simplifiable and level < 4):
#                     break
#
#         l = [fr1, op, fr2]
#         (cor, res, niveau) = OperateurPrioritaire(l, 4, solution=[])
#         if niveau >= 4:
#             break
#     fr1 = Fractions(n1, d1)
#     fr2 = Fractions(n2, d2)
#     return ([fr1, op, fr2], cor, res)
#
# def produits_fractions_4e(op, level):
#     '''Choisit des valeurs aléatoires pour effectuer un produit ou un quotient de
#     fractions en fonction du niveau de difficulté souhaité (de 1 é 4) et renvoie
#     l'énoncé et le corrigé au format TeX
#
#     @param op: '*' ou '/'
#     @param level: niveau de difficulté :
#                   1- Fractions positives non décomposables
#                   2- Fractions avec des nombres relatifs non décomposables
#                   3- Fractions positives é décomposer
#                   4- Fractions avec des nombres relatifs é décomposer
#     '''
#
#     while True:
#         (n1, d1, n2, d2) = (2, 2, 2, 2)
#         while True:
#             n1=d1=n2=d2=a=b=2
#             if level == 3 or level == 4:
#                 while Arithmetique.pgcd(a,b)>1:
#                     a=random.randrange(2,11)
#                     b=random.randrange(2,11)
#             else:
#                 a, b = 1, 1
#             if op == "*":
#                 if level == 1 or level == 3:
#                     while Arithmetique.pgcd(n1*a,d1*b)>1:
#                         n1=random.randrange(1,11)
#                         d1=random.randrange(2,11)
#                     while Arithmetique.pgcd(n2*b,d2*a)>1:
#                         n2=random.randrange(1,11)
#                         d2=random.randrange(2,11)
#                 elif level == 2 or level == 4:
#                     while True:
#                         neg=[(-1,-1,1)[random.randrange(3)] for x in range(4)]
#                         if neg[0]<0 or neg[1]<0 or neg[2]<0 or neg[3]<0:
#                             break
#                     while Arithmetique.pgcd(n1*a,d1*b)>1:
#                         n1=random.randrange(1,11)*neg[0]
#                         d1=random.randrange(2,11)*neg[1]
#                     while Arithmetique.pgcd(n2*b,d2*a)>1:
#                         n2=random.randrange(1,11)*neg[2]
#                         d2=random.randrange(2,11)*neg[3]
#                 fr1 = Fractions(n1*a, d1*b)
#                 fr2 = Fractions(n2*b, d2*a)
#                 simplifiable = abs(fr1.d * fr2.d) != Fractions.simplifie(fr1 *
#                                                                          fr2).d
#             else:
#                 if level == 1 or level == 3:
#                     while Arithmetique.pgcd(n1*a,d1*b)>1:
#                         n1=random.randrange(1,11)
#                         d1=random.randrange(2,11)
#                     while Arithmetique.pgcd(n2*a,d2*b)>1:
#                         n2=random.randrange(1,11)
#                         d2=random.randrange(2,11)
#                 else:
#                     while Arithmetique.pgcd(n1*a,d1*b)>1:
#                         n1=random.randrange(-11,11)
#                         d1=random.randrange(2,11)*(-1,1)[random.randrange(2)]
#                     while Arithmetique.pgcd(n2*a,d2*b)>1:
#                         n2=random.randrange(1,11)*(-1,1)[random.randrange(2)]
#                         d2=random.randrange(2,11)*(-1,1)[random.randrange(2)]
#                 fr1 = Fractions(n1*a, d1*b)
#                 fr2 = Fractions(n2*a, d2*b)
#                 simplifiable = abs(fr1.d * fr2.n) != \
#                                             Fractions.simplifie(fr1 / fr2).d
#             if (simplifiable and level>2) or (not simplifiable and level<=2):
#                 break
#         l = [fr1, op, fr2]
#
#         (cor, res, niveau) = OperateurPrioritaire(l, 4, solution=[])
#         if niveau >= 4:
#             break
#     return (l, cor, res)
#
#
# def valeurs_priorites_fractions(nb, entier=1):  # renvoie les 2 listes contenant les opérateurs et les opérandes.
#     listoperateurs = [
#         "+",
#         "*",
#         "-",
#         "/",
#         '(',
#         '(',
#         '(',
#         '(',
#         ')',
#         ')',
#         ')',
#         ')',
#         ]
#     loperateurs = []
#     loperandes = []
#     i = 0  #nombre d'opérateurs créés
#     p = 0  #nombre de parenthéses ouvertes
#     cpt = 0  #compteur pour éviter que le programme ne boucle.
#     while i < nb - 1:
#         cpt = cpt + 1
#         if cpt > 10:  #On recommence
#             (cpt, i, p, loperateurs) = (0, 0, 0, [])
#         if p:
#             if loperateurs[-1] == '(':  # On n'écrit pas 2 parenthéses é suivre
#                 operateur = listoperateurs[random.randrange(4)]
#             else:
#                 operateur = listoperateurs[random.randrange(12)]
#         elif loperateurs == []:
#
#             # On ne commence pas par une parenthése
#
#             operateur = listoperateurs[random.randrange(4)]
#         else:
#             operateur = listoperateurs[random.randrange(8)]
#         if nb > 3:
#             test = ('-*/').find(operateur) >= 0 and loperateurs.count(operateur) < \
#                 1 or operateur == "+" and loperateurs.count(operateur) < \
#                 2
#         else:
#             test = ('-*/+').find(operateur) >= 0 and loperateurs.count(operateur) < \
#                 1
#         if test:
#
#             #On n'accepte pas plus de 1 produit, différence, quotient et de 2 sommes ou parenthéses par calcul.
#
#             if i == 0 or loperateurs[-1] != '(' or ('*/').find(operateur) < \
#                 0:  #pas de * ou / dans une parenthése.
#                 i = i + 1
#                 loperateurs.append(operateur)
#         elif operateur == '(' and (')+').find(loperateurs[-1]) < 0:
#
#             #Il ne peut y avoir de ( aprés une ) ou aprés un +
#
#             p = p + 1
#             loperateurs.append(operateur)
#         elif operateur == ')':
#             p = p - 1
#             loperateurs.append(operateur)
#     while p > 0:
#         loperateurs.append(')')
#         p = p - 1
#     loperandes = []
#     for i in range(nb):
#         (n, d) = (2, 2)
#         while Arithmetique.pgcd(n, d) != 1 or abs(d) == 1:
#             n = Arithmetique.valeur_alea(-16, 16)
#             d = -Arithmetique.valeur_alea(-40, 40)
#         loperandes.append(Fractions(n, d))
#     exercice = [loperandes[0]]
#     i = 1
#     j = 0
#     while i < len(loperandes) or j < len(loperateurs):
#         if j < len(loperateurs):
#             exercice.append(loperateurs[j])
#             j = j + 1
#         while j < len(loperateurs) and (loperateurs[j] == '(' or
#                 loperateurs[j - 1] == ')'):
#             exercice.append(loperateurs[j])
#             j = j + 1
#         if i < len(loperandes):
#             exercice.append(loperandes[i])
#             i = i + 1
#     return exercice
#
#
# def exo_sommes_fractions():
#     exo = ["\\exercice",
#            u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
#            "\\begin{multicols}{4}", "  \\noindent%"]
#     cor = ["\\exercice*",
#            u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
#            "\\begin{multicols}{4}", "  \\noindent%"]
#     op = ["+", "-","+", "-","+", "-","+", "-"]
#     for i in range(8):
#         if i%2:
#             (l, sol, res) = sommes_fractions_4e(op.pop(0), i//2+1)
#         else:
#             (l, sol, res) = sommes_fractions_4e(op.pop(random.randrange(2)),
#                                                 i//2+1)
#         exo.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
#         cor.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
#         for l in sol:
#             if l == sol[-1]:
#                 cor.append("\\[\\boxed{\\thenocalcul = %s}\\]" % l)
#             else:
#                 cor.append("\\[\\thenocalcul = %s\\]" % l)
#         exo.append("\\stepcounter{nocalcul}%")
#         cor.append("\\stepcounter{nocalcul}%")
#     exo.append("\\end{multicols}\n")
#     cor.append("\\end{multicols}\n")
#     return (exo, cor)
#
# exo_sommes_fractions.description = u'Sommes de fractions'
#
#
# def exo_produits_fractions():
#     exo = ["\\exercice",
#            u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
#            "\\begin{multicols}{4}", "  \\noindent%"]
#     cor = ["\\exercice*",
#            u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
#            "\\begin{multicols}{4}", "  \\noindent%"]
#     op = ["*", "/","*", "/","*", "/","*", "/"]
#     for i in range(8):
#         if i%2:
#             (l, sol, res) = produits_fractions_4e(op.pop(0), i//2+1)
#         else:
#             (l, sol, res) = produits_fractions_4e(op.pop(random.randrange(2)), i//2+1)
#         exo.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
#         cor.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
#         for l in sol:
#             if l == sol[-1]:
#                 cor.append("\\[\\boxed{\\thenocalcul = %s}\\]" % l)
#             else:
#                 cor.append("\\[\\thenocalcul = %s\\]" % l)
#         exo.append("\\stepcounter{nocalcul}%")
#         cor.append("\\stepcounter{nocalcul}%")
#     exo.append("\\end{multicols}\n")
#     cor.append("\\end{multicols}\n")
#     return (exo, cor)
#
# exo_produits_fractions.description = u'Produits et quotients de fractions'
#
#
# def exo_priorites_fractions():
#     exo = ["\\exercice",
#            u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
#            "\\begin{multicols}{3}", "  \\noindent%"]
#     cor = ["\\exercice*",
#            u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
#            "\\begin{multicols}{3}", "  \\noindent%"]
#     for i in range(6):
#         while True:
#             l = valeurs_priorites_fractions(3)
#             (sol, res, niveau) = OperateurPrioritaire(l, 4, solution=[])
#             if niveau >= 4:
#                 break
#         exo.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
#         cor.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
#         for l in sol:
#             if l == sol[-1]:
#                 cor.append("\\[\\boxed{\\thenocalcul = %s}\\]" % l)
#             else:
#                 cor.append("\\[\\thenocalcul = %s\\]" % l)
#         exo.append("\\stepcounter{nocalcul}%")
#         cor.append("\\stepcounter{nocalcul}%")
#     exo.append("\\end{multicols}\n")
#     cor.append("\\end{multicols}\n")
#     return (exo, cor)
#
# exo_priorites_fractions.description = u'Fractions et priorités'
#
#
# exo_sommes_fractions()
#===============================================================================



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
'''
Created on 21 janv. 2015

@author: jerome
'''
from pyromaths import ex
from random import randrange
def listeToclasses(liste):
    result = [r'$[%s' % (liste[0])]
    for l in liste[1:]:
        result.append(r'$[%s' % (l + eval(result[-1][2:])))
        result[-2] = result[-2] + r' ~;~ %s[$' % (l + eval(result[-2][2:]))
    result.pop(-1)
    return result

class _Histogramme(ex.TexExercise):
    """Construire un histogramme"""

    tags = ["Seconde"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        echelle1 = 5 * 2 ** randrange(1, 6)
        debut = randrange(0, 4) * 10 ** randrange(0, 4)
        classes = [debut]
        classes.extend([echelle1 * randrange(1, 5) for dummy in range(randrange(4, 6))])
        echelle2 = 5 * 2 ** randrange(1, 6)
        effectifs = [echelle2 * randrange(1, 5) for dummy in range(len(classes) - 1)]
        self.classes = classes
        self.effectifs = effectifs
        self.echelle1 = echelle1
        self.echelle2 = echelle2

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(_(u'Tracer l\'histogramme de la série ci-dessous :\\par'))
        exo.append(r'\begin{tabularx}{.8\linewidth}[t]{|l|*{%s}{>{\centering\arraybackslash}X|}}' % len(self.classes))
        exo.append(_(r'\hline Classe & %s \\' % " & ").join(listeToclasses(self.classes)))
        exo.append(_(r'\hline Effectif & %s \\ \hline') % " & ".join([str(i) for i in self.effectifs]))
        exo.append(r'\end{tabularx}')
        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.append(r'\begin{center}')
        exo.append(r'\begin{asy}')
        exo.append(r'import stats;')
        exo.append(r'import graph;')
        exo.append(r'size(15cm,7cm,false);')
        # exo.append(r'unitsize(1cm);')
        exo.append(r'real[] tabxi={%s};' % ",".join([str(sum(self.classes[:i + 1])) for i in range(len(self.classes))]))
        exo.append(r'real[] tabni={%s};' % ",".join([str(i) for i in self.effectifs]))
        exo.append(r'real uniteaire=%s;' % self.echelle2)
        exo.append(r'string libelleunite="Effectif de %s.";' % (self.echelle2))
        exo.append(r'pen p1=lightred,p2=1bp+black;')
        exo.append(u'string libellecaractere="Valeurs du caractère";')
        exo.append(r'real minaxe=%s;' % self.classes[0])
        exo.append(r'real maxaxe=%s;' % sum(self.classes))
        exo.append(r'real uniteaxe=%s;' % self.echelle1)
        exo.append(r'real largeurunite=abs(tabxi[1]-tabxi[0]);')
        exo.append(r'int iclasse=0;')
        exo.append(r'real[] tabhi;')
        exo.append(r'for(int i=0; i < tabni.length; ++i){')
        exo.append(r'tabhi[i]=tabni[i]/(tabxi[i+1]-tabxi[i]);')
        exo.append(r'if (largeurunite>abs(tabxi[i+1]-tabxi[i])) {')
        exo.append(r'largeurunite=abs(tabxi[i+1]-tabxi[i]);')
        exo.append(r'iclasse=i;')
        exo.append(r'}')
        exo.append(r'}')
        exo.append(r'real hauteurmaxi=max(tabhi);')
        exo.append(r'real hauteurunite=(uniteaire/tabni[iclasse])*tabhi[iclasse];')
        exo.append(r'histogram(tabxi,tabhi,low=0,bars=true,p1,p2);')
        exo.append(r'filldraw(box((%s,hauteurmaxi+.5hauteurunite),(%s+largeurunite,hauteurmaxi+1.5hauteurunite)),p1,p2);' % (self.classes[0], self.classes[0]))
        exo.append(r'xaxis(libellecaractere, Bottom, minaxe,maxaxe,')
        exo.append(r'RightTicks(Label(currentpen+fontsize(6)), new real[]{%s}),above=false);' % (",".join([str(self.classes[0] + k * self.echelle1) for k in range(sum(self.classes) // self.echelle1 + 1)])))
        exo.append(r'for(int k=0; k<(2*hauteurmaxi/hauteurunite+4); ++k){')
        exo.append(r'draw((minaxe,.5k*hauteurunite)--(maxaxe,.5k*hauteurunite),.5bp+gray);')
        exo.append(r'}')
        exo.append(r'for(int k=0; k<=((maxaxe-minaxe)/uniteaxe); ++k){')
        exo.append(r'draw((minaxe+k*uniteaxe,0)--(minaxe+k*uniteaxe,hauteurmaxi+1.5hauteurunite),.5bp+gray);')
        exo.append(r'}')
        exo.append(r'label(libelleunite,(%s+largeurunite,hauteurmaxi+hauteurunite),align=E,Fill(white));' % (self.classes[0]))
        exo.append(r'histogram(tabxi,tabhi,low=0,bars=true,p1+opacity(0),p2);')
        exo.append(r'\end{asy}')
        exo.append(r'\end{center}')
        return "\n".join(exo)



#!/usr/bin/env python3
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
#----------------------------------------------------------------------
# Pyromaths : Poser des opérations
#----------------------------------------------------------------------

from pyromaths.outils import Arithmetique
from pyromaths.outils.Affichage import TeX, tex_coef
import random
from pyromaths.ex import LegacyExercise


def choix_trou(nb1, nb2, tot, operateur, exo, cor):
    nbaleatoire = random.randrange(4)
    if nbaleatoire > 1:
        exo.append("\\item $%s %s %s = \\ldots\\ldots\\ldots$" % (nb1,
                   operateur, nb2))
        cor.append("\\item $%s %s %s = \\mathbf{%s}$" % (nb1,
                   operateur, nb2, tot))
    elif nbaleatoire > 0:
        exo.append("\\item $%s %s \\ldots\\ldots\\ldots = %s$" % (nb1,
                   operateur, tot))
        cor.append("\\item $%s %s \\mathbf{%s} = %s$" % (nb1,
                   operateur, nb2, tot))
    else:
        exo.append("\\item $\\ldots\\ldots\\ldots %s %s = %s$" % (operateur,
                   nb2, tot))
        cor.append("\\item $\\mathbf{%s} %s %s = %s$" % (nb1,
                   operateur, nb2, tot))


def plus(pyromax):
    (a, b) = (Arithmetique.valeur_alea(-pyromax, pyromax), Arithmetique.valeur_alea(-pyromax,
              pyromax))
    return (a, b)


def moins(pyromax):
    (a, b) = (Arithmetique.valeur_alea(-pyromax, pyromax), Arithmetique.valeur_alea(-pyromax,
              pyromax))
    return (a + b, a)

def plus_dec(pyromax):
    (a, b) = (Arithmetique.valeur_alea(-10 * pyromax, 10 * pyromax) / 10, Arithmetique.valeur_alea(-10 * pyromax, 10 * pyromax) / 10)
    return (a, b)


def moins_dec(pyromax):
    (a, b) = (Arithmetique.valeur_alea(-10 * pyromax, 10 * pyromax) / 10, Arithmetique.valeur_alea(-10 * pyromax, 10 * pyromax) / 10)
    return (a + b, a)




def _relatifs():
    exo = ["\\exercice", _("Effectuer sans calculatrice :"),
           "\\begin{multicols}{3}\\noindent", "  \\begin{enumerate}"]
    cor = ["\\exercice*", _("Effectuer sans calculatrice :"),
           "\\begin{multicols}{3}\\noindent", "  \\begin{enumerate}"]
    modules = (plus, moins,)
    modules_dec = (plus_dec, moins_dec,)
    calculs = [i for i in range(20)]
    random.shuffle(calculs)
    for j in range(4):
        (a, b) = modules[calculs[j] // 10](10)
        choix_trou(a, tex_coef(b, '', bpn=1), a + b, '+', exo,
                       cor)

    for j in range(4, 14):
        (a, b) = modules[calculs[j] // 10](10)
        if calculs[j] // 10 == 0:
            choix_trou(a, tex_coef(b, '', bpn=1), a + b, '+', exo,
                       cor)
        if calculs[j] // 10 == 1:
            choix_trou(a, tex_coef(b, '', bpn=1), a - b, '-', exo,
                       cor)
    for j in range(14, 20):
        (a, b) = modules_dec[calculs[j] // 10](10)
        if calculs[j] // 10 == 0:
            choix_trou(TeX(a), tex_coef(b, '', bpn=1), TeX(a + b), '+', exo,
                       cor)
        if calculs[j] // 10 == 1:
            choix_trou(TeX(a), tex_coef(b, '', bpn=1), TeX(a - b), '-', exo,
                       cor)
    exo.extend(["  \\end{enumerate}", "\\end{multicols}"])
    cor.extend(["  \\end{enumerate}", "\\end{multicols}"])
    return (exo, cor)

class relatifs(LegacyExercise):
    """Addition de relatifs"""

    tags = ["Cinquième"]
    function = _relatifs



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from pyromaths.outils import Arithmetique
from . import fractions
from pyromaths.classes.PolynomesCollege import Polynome
from pyromaths.outils.Affichage import TeX, tex_coef
from pyromaths.outils.Priorites3 import texify
from pyromaths.ex import LegacyExercise

#
# ------------------- ÉQUATIONS -------------------


def valeurs(pyromax):  # crée les valeurs aléatoires pour l'équation
    while True:
        coefs = [Arithmetique.valeur_alea(-pyromax, pyromax) for i in range(6)]
        sgn = Arithmetique.valeur_alea(-1, 1)
        if sgn > 0:
            signe = "+"
        else:
            signe = "-"
        while True:
            while True:
                dens = [Arithmetique.valeur_alea(2, 9) for i in range(3)]
                if dens[0] != dens[1] and dens[0] != dens[2] and dens[1] != \
                    dens[2]:
                    break
            ppcm = Arithmetique.ppcm(dens[0], Arithmetique.ppcm(dens[1], dens[2]))
            densprim = [ppcm // dens[i] for i in range(3)]
            if densprim[0] < 10 and densprim[1] < 10 and densprim[2] < \
                10:
                break

        nvxcoefs = [coefs[i] * densprim[i // 2] for i in range(6)]
        if (nvxcoefs[0] + nvxcoefs[2] * sgn) - nvxcoefs[4] != 0:
            break
    return (tuple(coefs), tuple(dens), tuple(densprim), (signe, sgn),
            tuple(nvxcoefs))


def tex_quotient0(a, b, c):  # renvoie l'ecriture d'un quotient de l'enonce
    return '\\cfrac{%s}{%s}' % (str(Polynome('%sx+%s' % (a, b))), c)


def tex_quotient1(a, b, c, d):  # renvoie l'ecriture de la mise au meme denominateur d'un quotient
    if d == 1:
        return tex_quotient0(a, b, c)
    else:
        return '\\cfrac{(%s)_{\\times%s}}{%s_{\\times%s}}' % (str(Polynome([[a, 1], [b, 0]])), d, c, d)


def tex_equation0(valeurs):  # renvoie l'ecriture des quotients de l'enonce
    texte = ''
    for i in range(3):
        texte = texte + tex_quotient0(valeurs[0][i * 2], valeurs[0][i *
                2 + 1], valeurs[1][i])
        if i == 0:
            texte = texte + valeurs[3][0]
        elif i == 1:
            texte = texte + '='
    return texte


def tex_equation1(valeurs):  # renvoie l'ecriture de la mise au meme denominateur des quotients
    texte = ''
    for i in range(3):
        texte = texte + tex_quotient1(valeurs[0][i * 2], valeurs[0][i *
                2 + 1], valeurs[1][i], valeurs[2][i])
        if i == 0:
            texte = texte + valeurs[3][0]
        elif i == 1:
            texte = texte + '='
    return texte


def tex_equation2(valeurs):  # renvoie l'ecriture des quotients au meme denominateur
    texte = '\\cfrac{'
    texte += texify([['Polynome([[%s, 1], [%s, 0]])' % (valeurs[4][0], valeurs[4][1]), valeurs[3][0], 'Polynome([[%s, 1], [%s, 0]])' % (valeurs[4][2], valeurs[4][3])]])[0]
    texte = texte + '}{\\cancel{%s}}=\cfrac{' % (valeurs[1][0] * valeurs[2][0])
    texte += str(Polynome([[valeurs[4][4], 1] , [valeurs[4][5], 0]]))
    texte = texte + '}{\\cancel{%s}}' % (valeurs[1][0] * valeurs[2][0])
    return texte


def tex_equation2bis(valeurs):  # renvoie l'ecriture des quotients au meme denominateur sans les parentheses eventuelles
    texte = str(Polynome('%sx+%s' % (valeurs[4][0], valeurs[4][1])))
    texte = texte + str(Polynome('%sx+%s' % (valeurs[4][2] * valeurs[3][1], valeurs[4][3] * valeurs[3][1])))
    texte = texte + '=' + str(Polynome('%sx+%s' % (valeurs[4][4], valeurs[4][5])))
    return texte


def tex_equation3(valeurs):  # renvoie l'ecriture reduite de l'equation sans denominateur
    texte = str(Polynome('%sx+%s' % (valeurs[4][0] + valeurs[4][2] * valeurs[3][1], valeurs[4][1] + valeurs[4][3] * valeurs[3][1])))
    texte = texte + '=' + str(Polynome('%sx+%s' % (valeurs[4][4], valeurs[4][5])))
    return texte


def tex_equation4(valeurs):  # renvoie l'ecriture de l'equation avec l'inconnue d'un cote de l'egalite
    texte = tex_coef(valeurs[4][0] + valeurs[4][2] * valeurs[3][1], 'x') + tex_coef(-valeurs[4][4],
            'x', bplus=1)
    texte = texte + '=' + tex_coef(valeurs[4][5], '') + \
        tex_coef(-valeurs[4][1] - valeurs[4][3] * valeurs[3][1],
                                '', bplus=1)
    return texte


def tex_equation5(valeurs):  # renvoie l'ecriture reduite de l'equation avec l'inconnue d'un cote de l'egalite
    texte = tex_coef((valeurs[4][0] + valeurs[4][2] *
                                    valeurs[3][1]) - valeurs[4][4], 'x')
    texte = texte + '=' + tex_coef((valeurs[4][5] -
            valeurs[4][1]) - valeurs[4][3] * valeurs[3][1], '')
    return texte


def tex_equation6(valeurs):  # renvoie la solution de l'equation
    frac = ((valeurs[4][5] - valeurs[4][1]) - valeurs[4][3] * valeurs[3][1],
            (valeurs[4][0] + valeurs[4][2] * valeurs[3][1]) - valeurs[4][4])
    if (valeurs[4][0] + valeurs[4][2] * valeurs[3][1]) - valeurs[4][4] == \
        1:
        texte = ''
    else:
        texte = 'x=' + fractions.tex_frac(frac)
        simpl = fractions.simplifie(frac)
        if isinstance(simpl, tuple):
            texte = texte + '=' + fractions.tex_frac(simpl)
    return texte

tex_eqs = [tex_equation0, tex_equation1, tex_equation2, tex_equation3,
          tex_equation4, tex_equation5, tex_equation6]

def equations(exo, cor, valeurs):  # resolution d'une equation
    exo.append(u"Résoudre l'équation : ")
    exo.append(u'\\[ ' + tex_equation0(valeurs) + '\\] ')
    cor.append(u"Résoudre l'équation : ")
    for i in range(7):
        cor.append(u"\\[%s\\]" % tex_eqs[i](valeurs))
        if i == 2 and valeurs[3][1] < 0:
            cor.append(u'\\[ ' + tex_equation2bis(valeurs) + '\\] ')
    frac = ((valeurs[4][5] - valeurs[4][1]) - valeurs[4][3] * valeurs[3][1],
            (valeurs[4][0] + valeurs[4][2] * valeurs[3][1]) - valeurs[4][4])
    simpl = fractions.simplifie(frac)
    if isinstance(simpl, tuple):
        sol = fractions.tex_frac(simpl)
    else:
        sol = fractions.tex_frac(frac)
    cor.append(u'\\fbox{La solution de cette équation est $%s$\\,.}' %
             sol)

def _tex_equations():
    vals = valeurs(10)
    exo = ['\\exercice']
    cor = ['\\exercice*']
    equations(exo, cor, vals)
    return (exo, cor)

class tex_equations(LegacyExercise):
    """Équation"""

    tags = ["Troisième"]
    function = _tex_equations



#!/usr/bin/env python3
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#


import math
import random
from .Racine import simplifie_racine

class Vecteur:

    def __init__(self, x=0, y=0):
        self.x = x
        self.y = y

    def __eq__(self, vec):
        '''Teste l'égalité entre deux vecteurs'''
        return (self.x == vec.x) and (self.y == vec.y)

    def __add__(self, vec):
        '''Addition'''
        return Vecteur(self.x + vec.x, self.y + vec.y)

    def __sub__(self, vec):
        '''Soustraction'''
        return Vecteur(self.x - vec.x, self.y - vec.y)

    def __mul__(self, c):
        '''Multiplication par un nombre'''  # # Besoin du produit scalaire ou du produit vectoriel ?
        if isinstance(c, float) or isinstance(c, int):
            return  Vecteur(c * self.x, c * self.y)

    def __rmul__(self, c):
        '''Multiplication Reverse'''  # # Besoin du produit scalaire ou du produit vectoriel ?
        if isinstance(c, float) or isinstance(c, int):
            return  self * c
    def __neg__(self):
        '''Négatif d'un vecteur'''
        return -1 * self

    def __truediv__(self, c):
        if isinstance(c, float) or isinstance(c, int):
            return Vecteur(self.x / c, self.y / c)

    def __abs__(self):
        '''Retourne la norme du vecteur sous la forme coeff,radicande où sqrt(n)=coeff*sqrt(radicande)'''
        return simplifie_racine(self.x ** 2 + self.y ** 2)

    def __str__(self):
        '''Affichage sous forme de coordonnées'''
        return '(' + str(self.x) + ',' + str(self.y) + ')'

    def normeTex(self):
        '''Affichage TeX de la racine simplifiée de la racine, sans les dollars.'''
        norme = abs(self)
        if norme[0] == 1:
            return "\sqrt{" + str(norme[1]) + "}"
        elif norme[1] == 1:
            return str(norme[0])
        else:
            return str(norme[0]) + "\sqrt{" + str(norme[1]) + "}"

def randvect(a, b):
    '''Retourne un vecteur aléatoire et s'occupe de placer l'abscisse au-dessus de a et l'ordonnée dans [0,b]. Il faut que b>=10.'''
    x = random.randint(-5, 5)
    y = random.randint(-5, 5)
    posx = a + max(1 - x, 1)  # # 1 colonne minimum d'écart
    if y >= 0:
        posy = random.randint(0, math.fabs(y - 2)) + 1  # # Ajouter 1 pour éviter de coller au bas de la grille
    else:
        posy = random.randint(math.fabs(y), b)  # # Comme y < 0, cela ne colle pas au bas de la grille
    return [Vecteur(x, y), posx, posy]
